@@ -120,7 +120,7 @@ class FWAction(FWSerializable):
 
     def __init__(self, stored_data=None, exit=False, update_spec=None,
                  mod_spec=None, additions=None, detours=None,
-                 defuse_children=False):
+                 defuse_children=False, defuse_workflow=False):
         """
         :param stored_data: (dict) data to store from the run. Does not
         affect the operation of FireWorks.
@@ -134,6 +134,7 @@ class FWAction(FWSerializable):
         they will inherit the current FW's children)
         :param defuse_children: (bool) defuse all the original children of
         this Firework
+        :param defuse_workflow: (bool) defuse all incomplete steps of this workflow
         """
         mod_spec = mod_spec if mod_spec is not None else []
         additions = additions if additions is not None else []
@@ -147,6 +148,7 @@ class FWAction(FWSerializable):
             additions]
         self.detours = detours if isinstance(detours, (list, tuple)) else [detours]
         self.defuse_children = defuse_children
+        self.defuse_workflow = defuse_workflow
 
     @recursive_serialize
     def to_dict(self):
@@ -154,7 +156,8 @@ class FWAction(FWSerializable):
                 'update_spec': self.update_spec,
                 'mod_spec': self.mod_spec, 'additions': self.additions,
                 'detours': self.detours,
-                'defuse_children': self.defuse_children}
+                'defuse_children': self.defuse_children,
+                'defuse_workflow': self.defuse_workflow}
 
     @classmethod
     @recursive_deserialize
@@ -164,7 +167,7 @@ class FWAction(FWSerializable):
         detours = [Workflow.from_dict(f) for f in d['detours']]
         return FWAction(d['stored_data'], d['exit'], d['update_spec'],
                         d['mod_spec'], additions, detours,
-                        d['defuse_children'])
+                        d['defuse_children'], d.get('defuse_workflow', False))
 
     @property
     def skip_remaining_tasks(self):
@@ -174,7 +177,7 @@ class FWAction(FWSerializable):
 
         :return: (bool)
         """
-        return self.exit or self.detours or self.additions or self.defuse_children
+        return self.exit or self.detours or self.additions or self.defuse_children or self.defuse_workflow
 
     def __str__(self):
         return "FWAction\n" + pprint.pformat(self.to_dict())
@@ -262,6 +265,7 @@ class Firework(FWSerializable):
         if len(list(self.archived_launches)) > 0:
             m_dict['archived_launches'] = self.archived_launches
 
+        # keep export of new FWs to files clean
         if self.state != 'WAITING':
             m_dict['state'] = self.state
 
@@ -323,12 +327,6 @@ class Firework(FWSerializable):
     def __str__(self):
         return 'Firework object: (id: %i , name: %s)' % (self.fw_id, self.fw_name)
 
-# TODO: remove this (stop supporting) sometime around FW v1.0
-@deprecated(replacement=Firework)
-class FireWork(Firework):
-    pass
-
-
 class Tracker(FWSerializable, object):
     """
     A Tracker monitors a file and returns the last N lines for updating the Launch object
@@ -336,13 +334,14 @@ class Tracker(FWSerializable, object):
 
     MAX_TRACKER_LINES = 1000
 
-    def __init__(self, filename, nlines=TRACKER_LINES, content=''):
+    def __init__(self, filename, nlines=TRACKER_LINES, content='', allow_zipped=False):
         if nlines > self.MAX_TRACKER_LINES:
             raise ValueError("Tracker only supports a maximum of {} lines; you put {}.".format(
                 self.MAX_TRACKER_LINES, nlines))
         self.filename = filename
         self.nlines = nlines
         self.content = content
+        self.allow_zipped = allow_zipped
 
     def track_file(self, launch_dir=None):
         """
@@ -355,8 +354,11 @@ class Tracker(FWSerializable, object):
             m_file = os.path.join(launch_dir, m_file)
 
         lines = []
+        if self.allow_zipped:
+            m_file = zpath(m_file)
+
         if os.path.exists(m_file):
-            with zopen(zpath(m_file)) as f:
+            with zopen(m_file, "rt") as f:
                 for l in reverse_readline(f):
                     lines.append(l)
                     if len(lines) == self.nlines:
@@ -366,14 +368,16 @@ class Tracker(FWSerializable, object):
         return self.content
 
     def to_dict(self):
-        m_dict = {'filename': self.filename, 'nlines': self.nlines}
+        m_dict = {'filename': self.filename, 'nlines': self.nlines,
+                  'allow_zipped': self.allow_zipped}
         if self.content:
             m_dict['content'] = self.content
         return m_dict
 
     @classmethod
     def from_dict(cls, m_dict):
-        return Tracker(m_dict['filename'], m_dict['nlines'], m_dict.get('content', ''))
+        return Tracker(m_dict['filename'], m_dict['nlines'],
+                       m_dict.get('content', ''), m_dict.get('allow_zipped', False))
 
     def __str__(self):
         return '### Filename: {}\n{}'.format(self.filename, self.content)
@@ -601,7 +605,6 @@ class Workflow(FWSerializable):
                             pass  # garbage input
                     del self[k]
 
-
         @property
         def nodes(self):
             allnodes = list(self.keys())
@@ -661,7 +664,6 @@ class Workflow(FWSerializable):
         :param updated_on: (datetime)
         :param fw_states: (dict) - leave alone unless you are purposefully creating a Lazy-style WF
         """
-
         name = name or 'unnamed WF'  # prevent None names
 
         links_dict = links_dict if links_dict else {}
@@ -716,14 +718,41 @@ class Workflow(FWSerializable):
         m_state = 'READY'
         #states = [fw.state for fw in self.fws]
         states = self.fw_states.values()
-        if all([s == 'COMPLETED' for s in states]):
+        leaf_states = [self.fw_states[fw_id] for fw_id in self.leaf_fw_ids]
+        if all([s == 'COMPLETED' for s in leaf_states]):
             m_state = 'COMPLETED'
         elif all([s == 'ARCHIVED' for s in states]):
             m_state = 'ARCHIVED'
         elif any([s == 'DEFUSED' for s in states]):
             m_state = 'DEFUSED'
         elif any([s == 'FIZZLED' for s in states]):
-            m_state = 'FIZZLED'
+            # When _allow_fizzled_parents is set for some fireworks, the workflow is running if a given fizzled
+            # firework has all its childs COMPLETED, RUNNING, RESERVED or READY.
+            # For each fizzled fw, we thus have to check the states of their children
+            fizzled_ids = [fw_id for fw_id, state in self.fw_states.items() if state not in ['READY', 'RUNNING',
+                                                                                             'COMPLETED', 'RESERVED']]
+            for fizzled_id in fizzled_ids:
+                # If a fizzled fw is a leaf fw, then the workflow is fizzled
+                if fizzled_id in self.leaf_fw_ids:
+                    m_state = 'FIZZLED'
+                    break
+                childs_ids = self.links[fizzled_id]
+                mybreak = False
+                for child_id in childs_ids:
+                    # If one of the childs of a fizzled fw is also fizzled, then the workflow is fizzled
+                    # WARNING: this does not handle the case in which the childs of this child might be not fizzled
+                    #          one would need some recursive check here, but we can assume that _allow_fizzled_parents
+                    #          is usually not set twice in a row (in a child as well as in a "grandchild" of a given
+                    #          fw). Anyway, if in the end the workflow reaches completion, its state will be COMPLETED
+                    #          as it will be set as such by the first check on COMPLETED states of all leaf fireworks.
+                    if self.fw_states[child_id] == 'FIZZLED':
+                        mybreak = True
+                        m_state = 'FIZZLED'
+                        break
+                if mybreak:
+                    break
+            else:
+                m_state = 'RUNNING'
         elif any([s == 'COMPLETED' for s in states]) or any([s == 'RUNNING' for s in states]):
             m_state = 'RUNNING'
         elif any([s == 'RESERVED' for s in states]):
@@ -761,11 +790,18 @@ class Workflow(FWSerializable):
                 self.id_fw[cfid].state = 'DEFUSED'
                 updated_ids.append(cfid)
 
+        # defuse workflow
+        if action.defuse_workflow:
+            for fw_id in self.links.nodes:
+                if self.id_fw[fw_id].state not in ['FIZZLED', 'COMPLETED']:
+                    self.id_fw[fw_id].state = 'DEFUSED'
+                    updated_ids.append(fw_id)
+
         # add detour FireWorks
         # this should be done *before* additions
         if action.detours:
             for wf in action.detours:
-                new_updates = self.add_wf_to_fws(wf, [fw_id], pull_spec_mods=False, detour=True)
+                new_updates = self.append_wf(wf, [fw_id], detour=True, pull_spec_mods=False)
                 if len(set(updated_ids).intersection(new_updates)) > 0:
                     raise ValueError(
                         "Cannot use duplicated fw_ids when dynamically detouring workflows!")
@@ -774,7 +810,7 @@ class Workflow(FWSerializable):
         # add additional FireWorks
         if action.additions:
             for wf in action.additions:
-                new_updates = self.add_wf_to_fws(wf, [fw_id], pull_spec_mods=False, detour=False)
+                new_updates = self.append_wf(wf, [fw_id], detour=False, pull_spec_mods=False)
                 if len(set(updated_ids).intersection(new_updates)) > 0:
                     raise ValueError(
                         "Cannot use duplicated fw_ids when dynamically adding workflows!")
@@ -802,15 +838,15 @@ class Workflow(FWSerializable):
         # refresh the WF to get the states updated
         return self.refresh(fw_id, updated_ids)
 
-    def add_wf_to_fws(self, new_wf, fw_ids, pull_spec_mods=True, detour=False):
+    def append_wf(self, new_wf, fw_ids, detour=False, pull_spec_mods=False):
         """
-        Internal method to add a workflow as a child to a Firework
+        Method to add a workflow as a child to a Firework
         Note: detours must have children that have STATE_RANK that is WAITING or below
 
         :param new_wf: (Workflow) New Workflow to add
         :param fw_ids: ([int]) ids of the parent Fireworks on which to add the Workflow
-        :param pull_spec_mods: (bool) pull spec mods of COMPLETED parents
         :param detour: (bool) add children of the current Firework to the Workflow's leaves
+        :param pull_spec_mods: (bool) pull spec mods of COMPLETED parents, refreshes the WF states.
         :return: ([int]) list of Firework ids that were updated or new
         """
         updated_ids = []
@@ -818,6 +854,16 @@ class Workflow(FWSerializable):
         root_ids = new_wf.root_fw_ids
         leaf_ids = new_wf.leaf_fw_ids
 
+        # make sure detour runs do not link to ready/running/completed/etc. runs
+        if detour:
+            for fw_id in fw_ids:
+                if fw_id in self.links:
+                    # make sure all of these links are WAITING, else the DETOUR is not well defined
+                    ready_run = [(f >= 0 and Firework.STATE_RANKS[self.fw_states[f]] > 1) for f in self.links[fw_id]]
+                    if any(ready_run):
+                        raise ValueError("fw_id: {}: Detour option only works if all children of detours are not READY to run and have not already run".format(fw_id))
+
+        # make sure all new child fws have negative fw_id
         for new_fw in new_wf.fws:
             if new_fw.fw_id >= 0:  # note - this is also used later in the 'detour' code
                 raise ValueError(
@@ -825,25 +871,24 @@ class Workflow(FWSerializable):
                     '{}'.format(
                         new_fw.fw_id))
 
+        # completed checks - go ahead and append
+        for new_fw in new_wf.fws:
             self.id_fw[new_fw.fw_id] = new_fw  # add new_fw to id_fw
 
-            for fw_id in fw_ids:
-                if new_fw.fw_id in leaf_ids:
-                    if detour:
-                        # make sure all of these links are WAITING, else the DETOUR is not well defined
-                        ready_run = [(f >= 0 and Firework.STATE_RANKS[self.fw_states[f]] > 1) for f in self.links[fw_id]]
-                        if any(ready_run):
-                            raise ValueError("Detour option only works if all children of detours are not READY to run and have not already run")
+            if new_fw.fw_id in leaf_ids:
+                if detour:
+                    for fw_id in fw_ids:
                         self.links[new_fw.fw_id] = [f for f in self.links[fw_id] if f >= 0]  # add children of current FW to new FW
-                    else:
-                        self.links[new_fw.fw_id] = []
                 else:
-                    self.links[new_fw.fw_id] = new_wf.links[new_fw.fw_id]
-                updated_ids.append(new_fw.fw_id)
+                    self.links[new_fw.fw_id] = []
+            else:
+                self.links[new_fw.fw_id] = new_wf.links[new_fw.fw_id]
+            updated_ids.append(new_fw.fw_id)
 
         for fw_id in fw_ids:
             for root_id in root_ids:
                 self.links[fw_id].append(root_id)  # add the root id as my child
+
                 if pull_spec_mods:  # re-apply some actions of the parent
                     m_fw = self.id_fw[fw_id]  # get the parent FW
                     m_launch = self._get_representative_launch(m_fw)  # get Launch of parent
@@ -876,7 +921,6 @@ class Workflow(FWSerializable):
         fw = self.id_fw[fw_id]
         prev_state = fw.state
 
-
         # if we're defused or archived, just skip altogether
         if fw.state == 'DEFUSED' or fw.state == 'ARCHIVED':
             self.fw_states[fw_id] = fw.state
@@ -890,8 +934,7 @@ class Workflow(FWSerializable):
         if fw.spec.get('_allow_fizzled_parents'):
             completed_parent_states.append('FIZZLED')
 
-        if len(parent_states) != 0 and not all(
-                [s in completed_parent_states for s in parent_states]):
+        if len(parent_states) != 0 and not all(s in completed_parent_states for s in parent_states):
             m_state = 'WAITING'
 
         else:
@@ -921,8 +964,8 @@ class Workflow(FWSerializable):
                 updated_ids = updated_ids.union(self.apply_action(m_action,
                                                                   fw.fw_id))
 
-            # refresh all the children
-            for child_id in self.links[fw_id]:
+            # refresh all the children and other updated ids (note that defuse_workflow option can affect other branches)
+            for child_id in updated_ids.union(self.links[fw_id]):
                 updated_ids = updated_ids.union(
                     self.refresh(child_id, updated_ids))
 
@@ -987,7 +1030,7 @@ class Workflow(FWSerializable):
         return {'fws': [f.to_dict() for f in self.id_fw.values()],
                 'links': self.links.to_dict(),
                 'name': self.name,
-                'metadata': self.metadata, 'updated_on': self.updated_on}
+                'metadata': self.metadata, 'updated_on': self.updated_on, 'created_on': self.created_on}
 
     def to_db_dict(self):
         m_dict = self.links.to_db_dict()
@@ -1033,6 +1076,41 @@ class Workflow(FWSerializable):
 
         return m_launch
 
+    @classmethod
+    def from_wflow(cls, wflow):
+        """Create a fresh Workflow from an existing one.
+
+        """
+
+        new_wf = Workflow.from_dict(wflow.to_dict())
+        new_wf.reset(reset_ids=True)
+
+        return new_wf
+
+    def reset(self, reset_ids=True):
+        """Reset the states of all Fireworks in this workflow to 'WAITING'.
+
+        :param reset_ids: (bool) if ``True``, give each Firework a new id.
+
+        """
+        if reset_ids:
+            old_new = {} # mapping between old and new Firework ids
+            for fw_id, fw in self.id_fw.items():
+                global NEGATIVE_FWID_CTR
+                NEGATIVE_FWID_CTR -= 1
+                new_id = NEGATIVE_FWID_CTR
+
+                old_new[fw_id] = new_id
+                fw.fw_id = new_id
+        
+            self._reassign_ids(old_new)
+
+        # reset states
+        for fw in self.fws:
+            fw.state = 'WAITING'
+
+        self.fw_states = {key: self.id_fw[key].state for key in self.id_fw}
+
     @classmethod
     def from_dict(cls, m_dict):
         # accept either a Workflow dict or a Firework dict
@@ -1043,12 +1121,13 @@ class Workflow(FWSerializable):
                             Workflow.Links.from_dict(m_dict['links']), m_dict.get('name'),
                             m_dict['metadata'], created_on, updated_on)
         else:
-            return Workflow.from_FireWork(Firework.from_dict(m_dict))
+            return Workflow.from_Firework(Firework.from_dict(m_dict))
 
     @classmethod
-    def from_FireWork(cls, fw, name=None, metadata=None):
+    def from_Firework(cls, fw, name=None, metadata=None):
         name = name if name else fw.name
-        return Workflow([fw], None, name=name, metadata=metadata)
+        return Workflow([fw], None, name=name, metadata=metadata, created_on=fw.created_on,
+                        updated_on=fw.updated_on)
 
     def __str__(self):
         return 'Workflow object: (fw_ids: {} , name: {})'.format(self.id_fw.keys(), self.name)
