@@ -17,6 +17,7 @@ import distutils.dir_util
 from fireworks.core.firework import FWAction, Firework
 from fireworks.fw_config import FWData, PING_TIME_SECS, REMOVE_USELESS_DIRS, PRINT_FW_JSON, PRINT_FW_YAML, STORE_PACKING_INFO
 from fireworks.utilities.dict_mods import apply_mod
+from fireworks.core.launchpad import LockedWorkflowError
 
 __author__ = 'Anubhav Jain'
 __copyright__ = 'Copyright 2013, The Materials Project'
@@ -42,8 +43,7 @@ def start_ping_launch(launchpad, launch_id):
     if fd.MULTIPROCESSING:
         if not launch_id:
             raise ValueError("Multiprocessing cannot be run in offline mode!")
-        m = fd.DATASERVER
-        m.Running_IDs()[os.getpid()] = launch_id
+        fd.Running_IDs[os.getpid()] = launch_id
         return None
     else:
         ping_stop = threading.Event()
@@ -55,8 +55,7 @@ def start_ping_launch(launchpad, launch_id):
 def stop_backgrounds(ping_stop, btask_stops):
     fd = FWData()
     if fd.MULTIPROCESSING:
-        m = fd.DATASERVER
-        m.Running_IDs()[os.getpid()] = None
+        fd.Running_IDs[os.getpid()] = None
     else:
         ping_stop.set()
 
@@ -131,21 +130,14 @@ class Rocket():
             print("No FireWorks are ready to run and match query! {}".format(self.fworker.query))
             return False
 
-        if lp:
-            message = 'RUNNING fw_id: {} in directory: {}'.\
-                format(m_fw.fw_id, os.getcwd())
-            lp.log_message(logging.INFO, message)
-
-        # write FW.json and/or FW.yaml to the directory
-        if PRINT_FW_JSON:
-            m_fw.to_file('FW.json', indent=4)
-        if PRINT_FW_YAML:
-            m_fw.to_file('FW.yaml')
+        final_state = None
 
         try:
-            if '_launch_dir' in m_fw.spec:
+            if '_launch_dir' in m_fw.spec and lp:
                 prev_dir = launch_dir
                 launch_dir = os.path.expandvars(m_fw.spec['_launch_dir'])
+                if not os.path.abspath(launch_dir):
+                    launch_dir = os.path.normpath(os.path.join(os.getcwd(), launch_dir))
                 # thread-safe "mkdir -p"
                 try:
                     os.makedirs(launch_dir)
@@ -153,9 +145,8 @@ class Rocket():
                     if exception.errno != errno.EEXIST:
                         raise
                 os.chdir(launch_dir)
-                launch_dir = os.path.abspath(os.getcwd())
 
-                if lp:
+                if not os.path.samefile(launch_dir, prev_dir):
                     lp.change_launch_dir(launch_id, launch_dir)
 
                 if not os.listdir(prev_dir) and REMOVE_USELESS_DIRS:
@@ -182,6 +173,17 @@ class Rocket():
             else:
                 starting_task = 0
 
+            if lp:
+                message = 'RUNNING fw_id: {} in directory: {}'.\
+                    format(m_fw.fw_id, os.getcwd())
+                lp.log_message(logging.INFO, message)
+
+            # write FW.json and/or FW.yaml to the directory
+            if PRINT_FW_JSON:
+                m_fw.to_file('FW.json', indent=4)
+            if PRINT_FW_YAML:
+                m_fw.to_file('FW.yaml')
+
             my_spec = dict(m_fw.spec)  # make a copy of spec, don't override original
             my_spec["_fw_env"] = self.fworker.env
 
@@ -198,6 +200,13 @@ class Rocket():
             for t_counter, t in enumerate(m_fw.tasks[starting_task:], start=starting_task):
                 if lp:
                     lp.log_message(logging.INFO, "Task started: %s." % t.fw_name)
+
+                if my_spec.get("_add_launchpad_and_fw_id"):
+                    t.launchpad = self.launchpad
+                    t.fw_id = m_fw.fw_id
+                if my_spec.get("_add_fworker"):
+                    t.fworker = self.fworker
+
                 try:
                     m_action = t.run_task(my_spec)
                 except BaseException as e:
@@ -223,20 +232,23 @@ class Rocket():
                     m_action = FWAction(stored_data={'_message': 'runtime error during task', '_task': m_task,
                                                      '_exception': {'_stacktrace': tb, '_details': exception_details,
                                                                     '_failed_task_n': t_counter}}, exit=True)
+                    m_action = self.decorate_fwaction(m_action, my_spec, m_fw, launch_dir)
+
                     if lp:
-                        lp.complete_launch(launch_id, m_action, 'FIZZLED')
+                        final_state = 'FIZZLED'
+                        lp.complete_launch(launch_id, m_action, final_state)
                     else:
                         with open('FW_offline.json', 'r+') as f:
                             d = json.loads(f.read())
                             d['fwaction'] = m_action.to_dict()
                             d['state'] = 'FIZZLED'
+                            d['completed_on'] = datetime.utcnow().isoformat()
                             f.seek(0)
                             f.write(json.dumps(d))
                             f.truncate()
 
                     return True
 
-
                 # read in a FWAction from a file, in case the task is not Python and cannot return it explicitly
                 if os.path.exists('FWAction.json'):
                     m_action = FWAction.from_file('FWAction.json')
@@ -280,8 +292,11 @@ class Rocket():
             m_action.mod_spec = all_mod_spec
             m_action.update_spec = all_update_spec
 
+            m_action = self.decorate_fwaction(m_action, my_spec, m_fw, launch_dir)
+
             if lp:
-                lp.complete_launch(launch_id, m_action, 'COMPLETED')
+                final_state = 'COMPLETED'
+                lp.complete_launch(launch_id, m_action, final_state)
             else:
                 with open('FW_offline.json', 'r+') as f:
                     d = json.loads(f.read())
@@ -294,6 +309,13 @@ class Rocket():
 
             return True
 
+        except LockedWorkflowError as e:
+            lp.log_message(logging.DEBUG, traceback.format_exc())
+            lp.log_message(logging.WARNING, "Firework {} reached final state {} but couldn't complete the update of the"
+                                            " database. Reason: {}\nRefresh the WF to recover the result (lpad admin "
+                                            "refresh -i {}).".format(self.fw_id, final_state, e, self.fw_id))
+            return True
+
         except:
             # problems while processing the results. high probability of malformed data.
             traceback.print_exc()
@@ -307,15 +329,41 @@ class Rocket():
             m_action = FWAction(stored_data={'_message': 'runtime error during task', '_task': None,
                                              '_exception': {'_stacktrace': traceback.format_exc(),
                                              '_details': None}}, exit=True)
+
+            try:
+                m_action = self.decorate_fwaction(m_action, my_spec, m_fw, launch_dir)
+            except:
+                traceback.print_exc()
+
             if lp:
-                lp.complete_launch(launch_id, m_action, 'FIZZLED')
+                try:
+                    lp.complete_launch(launch_id, m_action, 'FIZZLED')
+                except LockedWorkflowError as e:
+                    lp.log_message(logging.DEBUG, traceback.format_exc())
+                    lp.log_message(logging.WARNING, "Firework {} fizzled but couldn't complete the update of the database."
+                                                    " Reason: {}\nRefresh the WF to recover the result (lpad admin "
+                                                    "refresh -i {}).".format(self.fw_id, final_state, e, self.fw_id))
+                    return True
             else:
                 with open('FW_offline.json', 'r+') as f:
                     d = json.loads(f.read())
                     d['fwaction'] = m_action.to_dict()
                     d['state'] = 'FIZZLED'
+                    d['completed_on'] = datetime.utcnow().isoformat()
                     f.seek(0)
                     f.write(json.dumps(d))
                     f.truncate()
 
-            return True
\ No newline at end of file
+            return True
+
+    def decorate_fwaction(self, fwaction, my_spec, m_fw, launch_dir):
+
+        if my_spec.get("_pass_job_info"):
+            job_info = list(my_spec.get("_job_info", []))
+            job_info.append({"fw_id": m_fw.fw_id, "name": m_fw.name, "launch_dir": launch_dir})
+            fwaction.mod_spec.append({"_push_all": {"_job_info": job_info}})
+
+        if my_spec.get("_preserve_fworker"):
+            fwaction.update_spec['_fworker'] = self.fworker.name
+
+        return fwaction
\ No newline at end of file
