@@ -1,6 +1,7 @@
 # coding: utf-8
 
 from __future__ import unicode_literals
+from datetime import datetime
 
 """
 This module contains methods for launching Rockets, both singly and in rapid-fire mode
@@ -8,7 +9,7 @@ This module contains methods for launching Rockets, both singly and in rapid-fir
 
 import os
 import time
-from fireworks.fw_config import RAPIDFIRE_SLEEP_SECS
+from fireworks.fw_config import RAPIDFIRE_SLEEP_SECS, FWORKER_LOC
 from fireworks.core.fworker import FWorker
 from fireworks.core.rocket import Rocket
 from fireworks.utilities.fw_utilities import get_fw_logger, create_datestamp_dir, log_multi
@@ -21,6 +22,17 @@ __email__ = 'ajain@lbl.gov'
 __date__ = 'Feb 22, 2013'
 
 
+def get_fworker(fworker):
+    if fworker:
+        my_fwkr = fworker
+    elif FWORKER_LOC:
+        my_fwkr = FWorker.from_file(FWORKER_LOC)
+    else:
+        my_fwkr = FWorker()
+
+    return my_fwkr
+
+
 def launch_rocket(launchpad, fworker=None, fw_id=None, strm_lvl='INFO'):
     """
     Run a single rocket in the current directory
@@ -29,7 +41,7 @@ def launch_rocket(launchpad, fworker=None, fw_id=None, strm_lvl='INFO'):
     :param fw_id: (int) if set, a particular Firework to run
     :param strm_lvl: (str) level at which to output logs to stdout
     """
-    fworker = fworker if fworker else FWorker()
+    fworker = get_fworker(fworker)
     l_dir = launchpad.get_logdir() if launchpad else None
     l_logger = get_fw_logger('rocket.launcher', l_dir=l_dir, stream_level=strm_lvl)
 
@@ -40,7 +52,8 @@ def launch_rocket(launchpad, fworker=None, fw_id=None, strm_lvl='INFO'):
     return rocket_ran
 
 
-def rapidfire(launchpad, fworker=None, m_dir=None, nlaunches=0, max_loops=-1, sleep_time=None, strm_lvl='INFO'):
+def rapidfire(launchpad, fworker=None, m_dir=None, nlaunches=0, max_loops=-1,
+              sleep_time=None, strm_lvl='INFO', timeout=None):
     """
     Keeps running Rockets in m_dir until we reach an error. Automatically creates subdirectories for each Rocket.
     Usually stops when we run out of FireWorks from the LaunchPad.
@@ -48,24 +61,27 @@ def rapidfire(launchpad, fworker=None, m_dir=None, nlaunches=0, max_loops=-1, sl
     :param launchpad: (LaunchPad)
     :param fworker: (FWorker object)
     :param m_dir: (str) the directory in which to loop Rocket running
-    :param nlaunches: (int) 0 means 'until completion', -1 or "infinite" means to loop forever
-    :param max_loops: (int) maximum number of loops
+    :param nlaunches: (int) 0 means 'until completion', -1 or "infinite" means to loop until max_loops
+    :param max_loops: (int) maximum number of loops (default -1 is infinite)
     :param sleep_time: (int) secs to sleep between rapidfire loop iterations
     :param strm_lvl: (str) level at which to output logs to stdout
+    :param timeout: (int) # of seconds after which to stop the rapidfire process
     """
 
     sleep_time = sleep_time if sleep_time else RAPIDFIRE_SLEEP_SECS
     curdir = m_dir if m_dir else os.getcwd()
     l_logger = get_fw_logger('rocket.launcher', l_dir=launchpad.get_logdir(), stream_level=strm_lvl)
     nlaunches = -1 if nlaunches == 'infinite' else int(nlaunches)
-    fworker = fworker if fworker else FWorker()
+    fworker = get_fworker(fworker)
 
     num_launched = 0
+    start_time = datetime.now()
     num_loops = 0
 
-    while num_loops != max_loops:
+    while num_loops != max_loops and (not timeout or (datetime.now() - start_time).total_seconds() < timeout):
         skip_check = False  # this is used to speed operation
-        while skip_check or launchpad.run_exists(fworker):
+        while (skip_check or launchpad.run_exists(fworker)) and \
+                (not timeout or (datetime.now() - start_time).total_seconds() < timeout):
             os.chdir(curdir)
             launcher_dir = create_datestamp_dir(curdir, l_logger, prefix='launcher_')
             os.chdir(launcher_dir)
@@ -89,3 +105,4 @@ def rapidfire(launchpad, fworker=None, m_dir=None, nlaunches=0, max_loops=-1, sl
         time.sleep(sleep_time)
         num_loops += 1
         log_multi(l_logger, 'Checking for FWs to run...'.format(sleep_time))
+    os.chdir(curdir)
