@@ -1,9 +1,3 @@
-#!/usr/bin/env python
-
-"""
-TODO: Modify module doc.
-"""
-
 from __future__ import division
 
 __author__ = "Shyue Ping Ong"
