@@ -21,9 +21,11 @@ import filecmp
 from fireworks import Firework, Workflow, LaunchPad, FWorker
 from fw_tutorials.dynamic_wf.addmod_task import AddModifyTask
 from fireworks.core.rocket_launcher import rapidfire, launch_rocket
+from fireworks.queue.queue_launcher import setup_offline_job
 from fireworks.user_objects.firetasks.script_task import ScriptTask, PyTask
-from fireworks.core.tests.tasks import ExceptionTestTask, ExecutionCounterTask
+from fireworks.core.tests.tasks import ExceptionTestTask, ExecutionCounterTask, SlowAdditionTask, WaitWFLockTask
 import fireworks.fw_config
+from monty.os import cd
 
 TESTDB_NAME = 'fireworks_unittest'
 MODULE_DIR = os.path.dirname(os.path.abspath(__file__))
@@ -76,10 +78,19 @@ class LaunchPadTest(unittest.TestCase):
         fw = Firework(ScriptTask.from_str('echo "hello"'), name="hello")
         wf = Workflow([fw], name='test_workflow')
         self.lp.add_wf(wf)
+        self.assertRaises(ValueError, self.lp.reset, '', False, 0)
+        self.assertEqual(self.lp.workflows.count(), 1)
         self.lp.reset('',require_password=False)
         self.assertFalse(self.lp.get_fw_ids())
         self.assertFalse(self.lp.get_wf_ids())
 
+        # test failsafe in a strict way
+        for x in range(30):
+            self.lp.add_wf(Workflow([Firework(ScriptTask.from_str('echo "hello"'))]))
+
+        self.assertRaises(ValueError, self.lp.reset, '')
+        self.lp.reset('', False, 100)  # reset back
+
     def test_pw_check(self):
         fw = Firework(ScriptTask.from_str('echo "hello"'), name="hello")
         self.lp.add_wf(fw)
@@ -272,9 +283,8 @@ class LaunchPadDefuseReigniteRerunArchiveDeleteTest(unittest.TestCase):
         launch_rocket(self.lp, self.fworker)
         print('----------\nafter launch rocket\n--------')
 
-
         # defuse Workflow containing Zeus
-        print('----------\nstaring defuse rocket\n--------')
+        print('----------\nstarting defuse rocket\n--------')
         self.lp.defuse_wf(self.zeus_fw_id)
         print('----------\nafter defuse rocket\n--------')
         defused_ids = self.lp.get_fw_ids({'state':'DEFUSED'})
@@ -428,18 +438,56 @@ class LaunchPadLostRunsDetectTest(unittest.TestCase):
                 raise ValueError("FW never starts running")
         rp.terminate() # Kill the rocket
 
-        l, f = self.lp.detect_lostruns(0.5, max_runtime=5, min_runtime=0)
+        l, f, i = self.lp.detect_lostruns(0.01, max_runtime=5, min_runtime=0)
         self.assertEqual((l, f), ([1], [1]))
         time.sleep(4)   # Wait double the expected exec time and test
-        l, f = self.lp.detect_lostruns(2)
+        l, f, i = self.lp.detect_lostruns(2)
         self.assertEqual((l, f), ([1], [1]))
 
-        l, f = self.lp.detect_lostruns(2, min_runtime=10)  # script did not run for 10 secs
+        l, f, i = self.lp.detect_lostruns(2, min_runtime=10)  # script did not run for 10 secs
         self.assertEqual((l, f), ([], []))
 
-        l, f = self.lp.detect_lostruns(2, max_runtime=-1)  # script ran more than -1 secs
+        l, f, i = self.lp.detect_lostruns(2, max_runtime=-1)  # script ran more than -1 secs
         self.assertEqual((l, f), ([], []))
 
+        l, f, i = self.lp.detect_lostruns(0.01, max_runtime=5, min_runtime=0, rerun=True)
+        self.assertEqual((l, f), ([1], [1]))
+        self.assertEqual(self.lp.get_fw_by_id(1).state, 'READY')
+
+
+    def test_detect_lostruns_defuse(self):
+        # Launch the timed firework in a separate process
+        class RocketProcess(Process):
+            def __init__(self, lpad, fworker):
+                super(self.__class__,self).__init__()
+                self.lpad = lpad
+                self.fworker = fworker
+
+            def run(self):
+                launch_rocket(self.lpad, self.fworker)
+
+        rp = RocketProcess(self.lp, self.fworker)
+        rp.start()
+
+        # Wait for fw to start
+        it = 0
+        while not any([f.state == 'RUNNING' for f in self.lp.get_wf_by_fw_id_lzyfw(self.fw_id).fws]):
+            time.sleep(1)   # Wait 1 sec
+            it += 1
+            if it == 10:
+                raise ValueError("FW never starts running")
+        rp.terminate() # Kill the rocket
+
+        l, f, i = self.lp.detect_lostruns(0.01)
+        self.assertEqual((l, f), ([1], [1]))
+
+        self.lp.defuse_fw(1)
+
+        l, f, i = self.lp.detect_lostruns(0.01, rerun=True)
+        self.assertEqual((l, f), ([1], []))
+        self.assertEqual(self.lp.get_fw_by_id(1).state, 'DEFUSED')
+
+
     def test_state_after_run_start(self):
         # Launch the timed firework in a separate process
         class RocketProcess(Process):
@@ -688,7 +736,7 @@ class WorkflowFireworkStatesTest(unittest.TestCase):
             self.assertEqual(fw_state, fw_cache_state)
 
     def test_rerun_timed_fws(self):
-        # Launch all firework in a separate process
+        # Launch all fireworks in a separate process
         class RapidfireProcess(Process):
             def __init__(self, lpad, fworker):
                 super(self.__class__,self).__init__()
@@ -709,8 +757,8 @@ class WorkflowFireworkStatesTest(unittest.TestCase):
             fw_cache_state = wf.fw_states[fw_id]
             self.assertEqual(fw_state, fw_cache_state)
 
-        # Detect lostruns
-        lost_lids, lost_fwids = self.lp.detect_lostruns(expiration_secs=0.5)
+        # Detect lost runs
+        lost_lids, lost_fwids, inconsistent_fwids = self.lp.detect_lostruns(expiration_secs=0.5)
         # Ensure the states are sync 
         wf = self.lp.get_wf_by_fw_id_lzyfw(self.zeus_fw_id)
         fws = wf.id_fw
@@ -724,8 +772,21 @@ class WorkflowFireworkStatesTest(unittest.TestCase):
             self.lp.rerun_fw(fw_id)
         rp = RapidfireProcess(self.lp, self.fworker)
         rp.start()
-        time.sleep(1)   # Wait 1 sec  
-        # Ensure the states are sync 
+        for i in range(20):
+            wf = self.lp.get_wf_by_fw_id_lzyfw(self.zeus_fw_id)
+            fws = wf.id_fw
+            if fws[self.zeus_fw_id].state == 'READY':
+                time.sleep(0.5)   # Wait 1 sec  
+            else:
+                break
+        else:
+            # Firework hasn't started yet. Waited too long.
+            rp.terminate()
+            return
+
+        time.sleep(1)
+
+        # Ensure the states are in sync 
         wf = self.lp.get_wf_by_fw_id_lzyfw(self.zeus_fw_id)
         fws = wf.id_fw
         for fw_id in wf.fw_states:
@@ -776,15 +837,18 @@ class LaunchPadRerunExceptionTest(unittest.TestCase):
 
     def test_except_details_on_rerun(self):
         rapidfire(self.lp, self.fworker, m_dir=MODULE_DIR)
+        self.assertEqual(os.getcwd(), MODULE_DIR)
         self.lp.rerun_fw(1)
         fw = self.lp.get_fw_by_id(1)
         self.assertEqual(fw.spec['_exception_details'], self.error_test_dict)
 
     def test_task_level_rerun(self):
         rapidfire(self.lp, self.fworker, m_dir=MODULE_DIR)
+        self.assertEqual(os.getcwd(), MODULE_DIR)
         self.lp.rerun_fws_task_level(1)
         self.lp.update_spec([1], {'skip_exception': True})
         rapidfire(self.lp, self.fworker, m_dir=MODULE_DIR)
+        self.assertEqual(os.getcwd(), MODULE_DIR)
         dirs = sorted(glob.glob(os.path.join(MODULE_DIR, "launcher_*")))
         self.assertEqual(self.lp.get_fw_by_id(1).state, 'COMPLETED')
         self.assertEqual(ExecutionCounterTask.exec_counter, 1)
@@ -793,9 +857,11 @@ class LaunchPadRerunExceptionTest(unittest.TestCase):
 
     def test_task_level_rerun_cp(self):
         rapidfire(self.lp, self.fworker, m_dir=MODULE_DIR)
+        self.assertEqual(os.getcwd(), MODULE_DIR)
         self.lp.rerun_fws_task_level(1, recover_mode="cp")
         self.lp.update_spec([1], {'skip_exception': True})
         rapidfire(self.lp, self.fworker, m_dir=MODULE_DIR)
+        self.assertEqual(os.getcwd(), MODULE_DIR)
         dirs = sorted(glob.glob(os.path.join(MODULE_DIR, "launcher_*")))
         self.assertEqual(self.lp.get_fw_by_id(1).state, 'COMPLETED')
         self.assertEqual(ExecutionCounterTask.exec_counter, 1)
@@ -804,14 +870,229 @@ class LaunchPadRerunExceptionTest(unittest.TestCase):
 
     def test_task_level_rerun_prev_dir(self):
         rapidfire(self.lp, self.fworker, m_dir=MODULE_DIR)
+        self.assertEqual(os.getcwd(), MODULE_DIR)
         self.lp.rerun_fws_task_level(1, recover_mode="prev_dir")
         self.lp.update_spec([1], {'skip_exception': True})
         rapidfire(self.lp, self.fworker, m_dir=MODULE_DIR)
         fw = self.lp.get_fw_by_id(1)
+        self.assertEqual(os.getcwd(), MODULE_DIR)
         self.assertEqual(fw.state, 'COMPLETED')
         self.assertEqual(fw.launches[0].launch_dir, fw.archived_launches[0].launch_dir)
         self.assertEqual(ExecutionCounterTask.exec_counter, 1)
         self.assertEqual(ExceptionTestTask.exec_counter, 2)
 
+
+class WFLockTest(unittest.TestCase):
+
+    @classmethod
+    def setUpClass(cls):
+        cls.lp = None
+        cls.fworker = FWorker()
+        try:
+            cls.lp = LaunchPad(name=TESTDB_NAME, strm_lvl='ERROR')
+            cls.lp.reset(password=None, require_password=False)
+        except:
+            raise unittest.SkipTest('MongoDB is not running in localhost:27017! Skipping tests.')
+
+    @classmethod
+    def tearDownClass(cls):
+        if cls.lp:
+            cls.lp.connection.drop_database(TESTDB_NAME)
+
+    def setUp(self):
+        # set the defaults in the init of wflock to break the lock quickly
+        fireworks.core.launchpad.WFLock(3, False).__init__.__func__.__defaults__= (3, False)
+
+        self.error_test_dict = {'error': 'description', 'error_code': 1}
+        fw_slow = Firework(SlowAdditionTask(), spec={'seconds': 10}, fw_id=1)
+        fw_fast = Firework(WaitWFLockTask(), fw_id=2, spec={'_add_launchpad_and_fw_id': True})
+        fw_child = Firework(ScriptTask.from_str('echo "child"'), fw_id=3)
+        wf = Workflow([fw_slow, fw_fast, fw_child], {fw_slow: fw_child, fw_fast: fw_child})
+        self.lp.add_wf(wf)
+
+        self.old_wd = os.getcwd()
+
+    def tearDown(self):
+        self.lp.reset(password=None, require_password=False)
+        # Delete launch locations
+        if os.path.exists(os.path.join('FW.json')):
+            os.remove('FW.json')
+        os.chdir(self.old_wd)
+        for ldir in glob.glob(os.path.join(MODULE_DIR, "launcher_*")):
+            shutil.rmtree(ldir)
+
+    def test_fix_db_inconsistencies_completed(self):
+        class RocketProcess(Process):
+            def __init__(self, lpad, fworker, fw_id):
+                super(self.__class__,self).__init__()
+                self.lpad = lpad
+                self.fworker = fworker
+                self.fw_id = fw_id
+
+            def run(self):
+                launch_rocket(self.lpad, self.fworker, fw_id=self.fw_id)
+        # Launch the slow firework in a separate process
+        rp = RocketProcess(self.lp, self.fworker, fw_id=1)
+        rp.start()
+
+        time.sleep(1)
+        launch_rocket(self.lp, self.fworker, fw_id=2)
+
+        # wait for the slow to complete
+        rp.join()
+
+        fast_fw = self.lp.get_fw_by_id(2)
+
+        if fast_fw.state == 'FIZZLED':
+            stacktrace = self.lp.launches.find_one(
+                {'fw_id': 2}, {'action.stored_data._exception._stacktrace': 1})['action']['stored_data']['_exception']['_stacktrace']
+            if 'SkipTest' in stacktrace:
+                self.skipTest("The test didn't run correctly")
+
+        self.assertEqual(fast_fw.state, 'RUNNING')
+
+        child_fw = self.lp.get_fw_by_id(3)
+
+        self.assertTrue("SlowAdditionTask" in child_fw.spec)
+        self.assertFalse("WaitWFLockTask" in child_fw.spec)
+
+
+
+        self.lp._refresh_wf(fw_id=2)
+
+        child_fw = self.lp.get_fw_by_id(3)
+
+        self.assertTrue("WaitWFLockTask" in child_fw.spec)
+
+        fast_fw = self.lp.get_fw_by_id(2)
+
+        self.assertEqual(fast_fw.state, 'COMPLETED')
+
+    def test_fix_db_inconsistencies_fizzled(self):
+
+        class RocketProcess(Process):
+            def __init__(self, lpad, fworker, fw_id):
+                super(self.__class__,self).__init__()
+                self.lpad = lpad
+                self.fworker = fworker
+                self.fw_id = fw_id
+
+            def run(self):
+                launch_rocket(self.lpad, self.fworker, fw_id=self.fw_id)
+
+        self.lp.update_spec([2], {'fizzle': True})
+
+        # Launch the slow firework in a separate process
+        rp = RocketProcess(self.lp, self.fworker, fw_id=1)
+        rp.start()
+
+        time.sleep(1)
+        launch_rocket(self.lp, self.fworker, fw_id=2)
+
+        # wait for the slow to complete
+        rp.join()
+
+        fast_fw = self.lp.get_fw_by_id(2)
+
+        if fast_fw.state == 'FIZZLED':
+            stacktrace = self.lp.launches.find_one(
+                {'fw_id': 2}, {'action.stored_data._exception._stacktrace': 1})['action']['stored_data']['_exception']['_stacktrace']
+            if 'SkipTest' in stacktrace:
+                self.skipTest("The test didn't run correctly")
+
+        self.assertEqual(fast_fw.state, 'RUNNING')
+
+        child_fw = self.lp.get_fw_by_id(3)
+
+        self.assertTrue("SlowAdditionTask" in child_fw.spec)
+        self.assertFalse("WaitWFLockTask" in child_fw.spec)
+
+        self.lp._refresh_wf(fw_id=2)
+
+        fast_fw = self.lp.get_fw_by_id(2)
+
+        self.assertEqual(fast_fw.state, 'FIZZLED')
+
+
+class LaunchPadOfflineTest(unittest.TestCase):
+
+    @classmethod
+    def setUpClass(cls):
+        cls.lp = None
+        cls.fworker = FWorker()
+        try:
+            cls.lp = LaunchPad(name=TESTDB_NAME, strm_lvl='ERROR')
+            cls.lp.reset(password=None, require_password=False)
+        except:
+            raise unittest.SkipTest('MongoDB is not running in localhost:27017! Skipping tests.')
+
+    @classmethod
+    def tearDownClass(cls):
+        if cls.lp:
+            cls.lp.connection.drop_database(TESTDB_NAME)
+
+    def setUp(self):
+        fireworks.core.firework.EXCEPT_DETAILS_ON_RERUN = True
+
+        self.error_test_dict = {'error': 'description', 'error_code': 1}
+        fw = Firework(ScriptTask.from_str(
+            'echo "test offline"',
+            {'store_stdout':True}), name="offline_fw", fw_id=1)
+        self.lp.add_wf(fw)
+
+        self.launch_dir = os.path.join(MODULE_DIR, "launcher_offline")
+        os.makedirs(self.launch_dir)
+
+        self.old_wd = os.getcwd()
+
+    def tearDown(self):
+        self.lp.reset(password=None, require_password=False)
+        # Delete launch locations
+        if os.path.exists(os.path.join('FW.json')):
+            os.remove('FW.json')
+        os.chdir(self.old_wd)
+        for ldir in glob.glob(os.path.join(MODULE_DIR, "launcher_*")):
+            shutil.rmtree(ldir, ignore_errors=True)
+
+    def test__recover_completed(self):
+        fw, launch_id = self.lp.reserve_fw(self.fworker, self.launch_dir)
+        fw = self.lp.get_fw_by_id(1)
+        with cd(self.launch_dir):
+            setup_offline_job(self.lp, fw, launch_id)
+
+            # launch rocket without launchpad to trigger offline mode
+            launch_rocket(launchpad=None, fworker=self.fworker, fw_id=1)
+
+        self.assertIsNone(self.lp.recover_offline(launch_id))
+
+        fw = self.lp.get_fw_by_id(launch_id)
+
+        self.assertEqual(fw.state, 'COMPLETED')
+
+
+    def test_recover_errors(self):
+        fw, launch_id = self.lp.reserve_fw(self.fworker, self.launch_dir)
+        fw = self.lp.get_fw_by_id(1)
+        with cd(self.launch_dir):
+            setup_offline_job(self.lp, fw, launch_id)
+
+        # remove the directory to cause an exception
+        shutil.rmtree(self.launch_dir)
+
+        # recover ignoring errors
+        self.assertIsNotNone(self.lp.recover_offline(launch_id, ignore_errors=True, print_errors=True))
+
+        fw = self.lp.get_fw_by_id(launch_id)
+
+        self.assertEqual(fw.state, 'RESERVED')
+
+        #fizzle
+        self.assertIsNotNone(self.lp.recover_offline(launch_id, ignore_errors=False))
+
+        fw = self.lp.get_fw_by_id(launch_id)
+
+        self.assertEqual(fw.state, 'FIZZLED')
+                
+
 if __name__ == '__main__':
     unittest.main()
