@@ -44,6 +44,7 @@ FW_LOGGING_FORMAT = '%(asctime)s %(levelname)s %(message)s'  # format for logger
 
 QUEUE_RETRY_ATTEMPTS = 10  # number of attempts to re-try communicating with queue server in failures
 QUEUE_UPDATE_INTERVAL = 5  # max interval (seconds) needed for queue to update after submitting a job
+QUEUE_JOBNAME_MAXLEN = 20  # max length of the jobname for queue systems
 
 SUBMIT_SCRIPT_NAME = 'FW_submit.script'  # name of submit script
 
@@ -59,13 +60,13 @@ RESERVATION_EXPIRATION_SECS = 60 * 60 * 24 * 14  # a job can stay in a queue thi
 # cancel its reservation
 
 WFLOCK_EXPIRATION_SECS = 60 * 5  # wait this long for a WFLock before expiring
-WFLOCK_EXPIRATION_KILL = True  # kill WFLock on expiration (or give a warning)
+WFLOCK_EXPIRATION_KILL = False  # kill WFLock on expiration (or give a warning)
 
 RAPIDFIRE_SLEEP_SECS = 60  # seconds to sleep between rapidfire loops
 
 LAUNCHPAD_LOC = None  # where to find the my_launchpad.yaml file
 FWORKER_LOC = None  # where to find the my_fworker.yaml file
-QUEUEADAPTER_LOC = None  # where to find the my_queueadapter.yaml file
+QUEUEADAPTER_LOC = None  # where to find the my_qadapter.yaml file
 
 CONFIG_FILE_DIR = '.'  # directory containing config files (if not individually set)
 
@@ -87,8 +88,19 @@ TRACKER_LINES = 25  # number of lines to return in Tracker
 
 SORT_FWS = ''  # sort equal priority FWs? "FILO" or "FIFO".
 
+ENCODE_MONTY = True  # detect and use Monty-style as_dict()
+
+DECODE_MONTY = True  # detect and use Monty-style from_dict() with @class and @module
+
 EXCEPT_DETAILS_ON_RERUN = False  # add exception details to the spec when rerunning the FW
 
+WEBSERVER_HOST = "127.0.0.1"  # default host on which the Flask web server runs
+
+WEBSERVER_PORT = 5000  # default port on which the Flask web server runs
+
+MONGO_SOCKET_TIMEOUT_MS = 5 * 60 * 1000  # value of socketTimeoutMS when connection to mongoDB.  See pymongo official
+                                         # documentation http://api.mongodb.org/python/current/api/pymongo/mongo_client.html
+
 
 def override_user_settings():
     module_dir = os.path.dirname(os.path.abspath(__file__))
@@ -130,7 +142,8 @@ def override_user_settings():
 
     for k in ["LAUNCHPAD_LOC", "FWORKER_LOC", "QUEUEADAPTER_LOC"]:
         if globals().get(k, None) is None:
-            fname = "my_{}.yaml".format(k.split("_")[0].lower())
+            fname = "my_qadapter.yaml" if k == "QUEUEADAPTER_LOC" else \
+                "my_{}.yaml".format(k.split("_")[0].lower())
             m_paths = []
             if os.path.realpath(CONFIG_FILE_DIR) not in test_paths:
                 test_paths.insert(0, CONFIG_FILE_DIR)
@@ -175,3 +188,4 @@ class FWData(object):
         self.NODE_LIST = None  # the node list for sub jobs
         self.SUB_NPROCS = None  # the number of process of the sub job
         self.DATASERVER = None  # the shared object manager
+        self.Running_IDs = None
