@@ -1,6 +1,8 @@
 # coding: utf-8
 
 from __future__ import unicode_literals
+from fireworks.features.fw_report import FWReport
+from fireworks.features.introspect import Introspector
 
 """
 A runnable script for managing a FireWorks database (a command-line interface to launchpad.py)
@@ -19,14 +21,14 @@ import yaml
 
 from fireworks.fw_config import RESERVATION_EXPIRATION_SECS, \
     RUN_EXPIRATION_SECS, PW_CHECK_NUM, MAINTAIN_INTERVAL, CONFIG_FILE_DIR, \
-    LAUNCHPAD_LOC
-from fireworks.core.launchpad import LaunchPad
+    LAUNCHPAD_LOC, FWORKER_LOC, WEBSERVER_PORT, WEBSERVER_HOST
+from fireworks.core.launchpad import LaunchPad, WFLock
 from fireworks.core.firework import Workflow, Firework
+from fireworks.core.fworker import FWorker
 from fireworks import __version__ as FW_VERSION
 from fireworks import FW_INSTALL_DIR
 from fireworks.user_objects.firetasks.script_task import ScriptTask
 from fireworks.utilities.fw_serializers import DATETIME_HANDLER, recursive_dict
-from fireworks.features.stats import FWStats
 from six.moves import input
 
 
@@ -71,15 +73,24 @@ def parse_helper(lp, args, wf_mode=False, skip_pw=False):
         return pw_check(args.fw_id, args, skip_pw)
     if args.query:
         query = ast.literal_eval(args.query)
-    if args.name:
+    if args.name and not args.launches_mode:
         query['name'] = args.name
     if args.state:
         query['state'] = args.state
 
-    if wf_mode:
-        return pw_check(lp.get_wf_ids(query), args, skip_pw)
+    if hasattr(args, "sort") and args.sort:
+        sort = [(args.sort, ASCENDING)]
+    elif hasattr(args, "sort") and args.rsort:
+        sort = [(args.rsort, DESCENDING)]
+    else:
+        sort = None
 
-    return pw_check(lp.get_fw_ids(query), args, skip_pw)
+    max = args.max if hasattr(args, "max") else 0
+
+    if wf_mode:
+        return pw_check(lp.get_wf_ids(query, sort=sort, limit=max), args, skip_pw)
+
+    return pw_check(lp.get_fw_ids(query, sort=sort, limit=max, launches_mode=args.launches_mode), args, skip_pw)
 
 
 def get_lp(args):
@@ -106,7 +117,8 @@ def init_yaml(args):
         ("port", 27017),
         ("name", "fireworks"),
         ("username", None),
-        ("password", None))
+        ("password", None),
+        ("ssl_ca_file", None))
     doc = {}
     print("Please supply the following configuration values")
     print("(press Enter if you want to accept the defaults)\n")
@@ -116,7 +128,9 @@ def init_yaml(args):
     doc["port"] = int(doc["port"])  # enforce the port as an int
     with open(args.config_file, "w") as f:
         import yaml
-        yaml.dump(LaunchPad.from_dict(doc).to_dict(), f)
+        doc = LaunchPad.from_dict(doc).to_dict()
+        doc = recursive_dict(doc, preserve_unicode=False)  # drop unicode
+        yaml.dump(doc, f)
         print("\nConfiguration written to {}!".format(args.config_file))
 
 
@@ -160,7 +174,7 @@ def get_fws(args):
 
     if args.fw_id:
         query = {'fw_id': {"$in": args.fw_id}}
-    elif args.name:
+    elif args.name and not args.launches_mode:
         query = {'name': args.name}
     elif args.state:
         query = {'state': args.state}
@@ -180,10 +194,10 @@ def get_fws(args):
         ids = lp.get_fw_ids_from_reservation_id(args.qid)
         if query:
             query['fw_id'] = {"$in": ids}
-            ids = lp.get_fw_ids(query, sort, args.max)
+            ids = lp.get_fw_ids(query, sort, args.max, launches_mode=args.launches_mode)
 
     else:
-        ids = lp.get_fw_ids(query, sort, args.max, count_only=args.display_format == 'count')
+        ids = lp.get_fw_ids(query, sort, args.max, count_only=args.display_format == 'count', launches_mode=args.launches_mode)
     fws = []
     if args.display_format == 'ids':
         fws = ids
@@ -193,6 +207,7 @@ def get_fws(args):
         for id in ids:
             fw = lp.get_fw_by_id(id)
             d = fw.to_dict()
+            d['state'] = d.get('state', 'WAITING')
             if args.display_format == 'more' or args.display_format == 'less':
                 if 'archived_launches' in d:
                     del d['archived_launches']
@@ -284,26 +299,17 @@ def get_children(links, start, max_depth):
                 data[l] = c
     return data
 
-def display_wfs(args):
-    lp = get_lp(args)
-
-    query = {'nodes': {"$in": args.fw_id}}
-
-    ids = lp.get_wf_ids(query, None, 0, count_only=False)
-    for i in ids:
-        d = lp.get_wf_summary_dict(i, "all")
-
-        get_fwid = lambda l: int(l.split("--")[-1])
-        links = {get_fwid(k): [get_fwid(i) for i in v]
-                 for k, v in d["links"].items()}
-        c = get_children(links, i, 3)
-        print(args.output(c))
-
 def detect_lostruns(args):
     lp = get_lp(args)
-    fl, ff = lp.detect_lostruns(expiration_secs=args.time, fizzle=args.fizzle, rerun=args.rerun, max_runtime=args.max_runtime, min_runtime=args.min_runtime)
-    lp.m_logger.debug('Detected {} FIZZLED launches: {}'.format(len(fl), fl))
-    lp.m_logger.info('Detected {} FIZZLED FWs: {}'.format(len(ff), ff))
+    fl, ff, fi = lp.detect_lostruns(expiration_secs=args.time, fizzle=args.fizzle, rerun=args.rerun, max_runtime=args.max_runtime,
+                                min_runtime=args.min_runtime, refresh=args.refresh)
+    lp.m_logger.debug('Detected {} lost launches: {}'.format(len(fl), fl))
+    lp.m_logger.info('Detected {} lost FWs: {}'.format(len(ff), ff))
+    lp.m_logger.info('Detected {} inconsistent FWs: {}'.format(len(fi), fi))
+    if len(ff) > 0 and not args.fizzle and not args.rerun:
+        print("You can fix lost FWs using the --rerun or --fizzle arguments to the detect_lostruns command")
+    if len(fi) > 0 and not args.refresh:
+        print("You can fix inconsistent FWs using the --refresh argument to the detect_lostruns command")
 
 
 def detect_unreserved(args):
@@ -320,9 +326,12 @@ def defuse_wfs(args):
     lp = get_lp(args)
     fw_ids = parse_helper(lp, args, wf_mode=True)
     for f in fw_ids:
-        lp.defuse_wf(f)
+        lp.defuse_wf(f, defuse_all_states=args.defuse_all_states)
         lp.m_logger.debug('Processed fw_id: {}'.format(f))
-    lp.m_logger.info('Finished defusing {} FWs'.format(len(fw_ids)))
+    lp.m_logger.info('Finished defusing {} FWs.'.format(len(fw_ids)))
+    if not args.defuse_all_states:
+        lp.m_logger.info('Note: FIZZLED and COMPLETED FWs were not defused. Use the --defuse_all_states option to force this (or rerun FIZZLED FWs first).')
+
 
 
 def archive(args):
@@ -384,11 +393,23 @@ def refresh(args):
     lp = get_lp(args)
     fw_ids = parse_helper(lp, args, wf_mode=True)
     for f in fw_ids:
-        wf = lp.get_wf_by_fw_id(f)
-        lp._refresh_wf(wf, f)
+        wf = lp.get_wf_by_fw_id_lzyfw(f)
+        for fw_id in wf.root_fw_ids:
+            lp._refresh_wf(fw_id)
         lp.m_logger.debug('Processed Workflow with fw_id: {}'.format(f))
     lp.m_logger.info('Finished refreshing {} Workflows'.format(len(fw_ids)))
 
+
+def unlock(args):
+    lp = get_lp(args)
+    fw_ids = parse_helper(lp, args, wf_mode=True)
+    for f in fw_ids:
+        with WFLock(lp, f, expire_secs=0, kill=True):
+            lp.m_logger.warn('FORCIBLY RELEASING LOCK DUE TO USER COMMAND, WF: {}'.format(f))
+            lp.m_logger.debug('Processed Workflow with fw_id: {}'.format(f))
+    lp.m_logger.info('Finished unlocking {} Workflows'.format(len(fw_ids)))
+
+
 def get_qid(args):
     lp = get_lp(args)
     for f in args.fw_id:
@@ -411,14 +432,37 @@ def set_priority(args):
 def webgui(args):
     os.environ["FWDB_CONFIG"] = json.dumps(get_lp(args).to_dict())
     from fireworks.flask_site.app import app
-    from multiprocessing import Process
-    p1 = Process(target=app.run, args=())
-    p1.start()
+    if args.wflowquery:
+        app.BASE_Q_WF = json.loads(args.wflowquery)
+    if args.fwquery:
+        app.BASE_Q = json.loads(args.fwquery)
+        if "state" in app.BASE_Q:
+            app.BASE_Q_WF["state"] = app.BASE_Q["state"]
+
     if not args.server_mode:
+        from multiprocessing import Process
+        p1 = Process(
+            target=app.run,
+            kwargs={"host": args.host, "port": args.port, "debug": args.debug})
+        p1.start()
         import webbrowser
         time.sleep(2)
         webbrowser.open("http://{}:{}".format(args.host, args.port))
-    p1.join()
+        p1.join()
+    else:
+        from fireworks.flask_site.app import bootstrap_app
+        try:
+            from fireworks.flask_site.gunicorn import (
+                StandaloneApplication, number_of_workers)
+        except ImportError:
+            import sys
+            sys.exit("Gunicorn is required for server mode. "
+                     "Install using `pip install guncorn`.")
+        options = {
+            'bind': '%s:%s' % (args.host, args.port),
+            'workers': number_of_workers(),
+        }
+        StandaloneApplication(bootstrap_app, options).run()
 
 def add_scripts(args):
     lp = get_lp(args)
@@ -436,13 +480,20 @@ def add_scripts(args):
 
 def recover_offline(args):
     lp = get_lp(args)
+    fworker_name = FWorker.from_file(args.fworker_file).name if args.fworker_file else None
     failed_fws = []
-    for l in lp.offline_runs.find({"completed": False, "deprecated": False}, {"launch_id": 1}):
-        fw = lp.recover_offline(l['launch_id'], args.ignore_errors)
-        if fw:
-            failed_fws.append(fw)
+    recovered_fws = []
 
-    lp.m_logger.info("FINISHED recovering offline runs.")
+    for l in lp.offline_runs.find({"completed": False, "deprecated": False}, {"launch_id": 1, "fw_id":1}):
+        if fworker_name and lp.launches.count({"launch_id": l["launch_id"], "fworker.name": fworker_name}) == 0:
+            continue
+        fw = lp.recover_offline(l['launch_id'], args.ignore_errors, args.print_errors)
+        if fw:
+            failed_fws.append(l['fw_id'])
+        else:
+            recovered_fws.append(l['fw_id'])
+
+    lp.m_logger.info("FINISHED recovering offline runs. {} job(s) recovered: {}".format(len(recovered_fws), recovered_fws))
     if failed_fws:
         lp.m_logger.info("FAILED to recover offline fw_ids: {}".format(failed_fws))
 
@@ -458,52 +509,29 @@ def forget_offline(args):
 
 def report(args):
     lp=get_lp(args)
-    def print_results(results):
-        for i in results:
-            print('{}: {}'.format(i["_id"], str(i['count'])))
-            for k in i.keys():
-                if k not in ["_id", "count", "ids"]:
-                    print(' {}: {}'.format(k, str(i[k])))
-            if "ids" in i.keys():
-                print(" ids:"),
-                print(i["ids"])
-    def print_daily_results(results):
-        for i in results:
-            print(i["_id"])
-            for k in i["run_counts"]:
-                print(" {}: {}".format(k["state"], k["count"]))
-    if args.action_command=="fws":
-        results=FWStats(lp).get_fireworks_summary(query_start=args.start, query_end=args.end, query=args.query,
-                                                  time_field=args.time_field, weeks=args.weeks, days=args.days,
-                                                  hours=args.hours, minutes=args.minutes)
-        print_results(results)
-    elif args.action_command=="launches":
-        results=FWStats(lp).get_launch_summary(query_start=args.start, query_end=args.end, query=args.query,
-                                               time_field=args.time_field, weeks=args.weeks, days=args.days,
-                                               hours=args.hours, minutes=args.minutes, runtime_stats=args.runtime_stats,
-                                               include_ids=args.include_ids)
-        print_results(results)
-    elif args.action_command=="wfs":
-        results=FWStats(lp).get_workflow_summary(query_start=args.start, query_end=args.end, query=args.query,
-                                               time_field=args.time_field, weeks=args.weeks, days=args.days,
-                                               hours=args.hours, minutes=args.minutes)
-        print_results(results)
-    elif args.action_command=="daily":
-        results=FWStats(lp).get_daily_completion_summary(query_start=args.start, query_end=args.end, query=args.query,
-                                               time_field=args.time_field, weeks=args.weeks, days=args.days,
-                                               hours=args.hours, minutes=args.minutes)
-        print_daily_results(results)
-    elif args.action_command=="group_fizzled_fws":
-        results=FWStats(lp).group_fizzled_fireworks(group_by=args.group_by, query_start=args.start, query_end=args.end,
-                                                    query=args.query, weeks=args.weeks, days=args.days, hours=args.hours,
-                                                    minutes=args.minutes, include_ids=args.include_ids)
-        print_results(results)
-    elif args.action_command=="catastrophes":
-        results=FWStats(lp).identify_catastrophes(error_ratio=args.error_ratio, query_start=args.start, query_end=args.end,
-                                                  query=args.query, weeks=args.weeks, days=args.days,hours=args.hours,
-                                                  minutes=args.minutes,runtime_stats=args.runtime_stats,
-                                                  include_ids=args.include_ids)
-        print(results)
+    query = ast.literal_eval(args.query) if args.query else None
+    fwr = FWReport(lp)
+    stats = fwr.get_stats(coll=args.collection, interval=args.interval, num_intervals=args.num_intervals, additional_query=query)
+    title_str = "Stats on {}".format(args.collection)
+    title_dec = "-" * len(title_str)
+    print(title_dec)
+    print(title_str)
+    print(title_dec)
+    print(fwr.get_stats_str(stats))
+
+
+def introspect(args):
+    print("NOTE: This feature is in beta mode...")
+    lp=get_lp(args)
+
+    isp = Introspector(lp)
+    for coll in ['launches', 'tasks', 'fireworks', 'workflows']:
+        print('generating report for {}...please wait...'.format(coll))
+        print('')
+        table = isp.introspect_fizzled(coll=coll, threshold=args.threshold, limit=args.max)
+        isp.print_report(table, coll)
+        print('')
+
 
 def track_fws(args):
     lp = get_lp(args)
@@ -580,6 +608,11 @@ def lpad():
     query_kwargs = {"help": 'Query (enclose pymongo-style dict in '
                             'single-quotes, e.g. \'{"state":"COMPLETED"}\')'}
 
+    launches_mode_args = ["-lm", "--launches_mode"]
+    launches_mode_kwargs = {"action": "store_true",
+                            "help": 'Query the launches collection (enclose pymongo-style '
+                                    'dict in single-quotes, e.g. \'{"launch_id": 1}\')'}
+
     qid_args = ["--qid"]
     qid_kwargs = {"help": "Query by reservation id of job in queue"}
 
@@ -621,6 +654,7 @@ def lpad():
     get_fw_parser.add_argument('-n', '--name', help='get FWs with this name')
     get_fw_parser.add_argument(*state_args, **state_kwargs)
     get_fw_parser.add_argument(*query_args, **query_kwargs)
+    get_fw_parser.add_argument(*launches_mode_args, **launches_mode_kwargs)
     get_fw_parser.add_argument(*qid_args, **qid_kwargs)
     get_fw_parser.add_argument(*disp_args, **disp_kwargs)
     get_fw_parser.add_argument('-m', '--max', help='limit results', default=0,
@@ -637,10 +671,12 @@ def lpad():
     trackfw_parser.add_argument('-n', '--name', help='name')
     trackfw_parser.add_argument(*state_args, **state_kwargs)
     trackfw_parser.add_argument(*query_args, **query_kwargs)
+    trackfw_parser.add_argument(*launches_mode_args, **launches_mode_kwargs)
     trackfw_parser.add_argument('-c', '--include', nargs="+",
                                 help='only include these files in the report')
     trackfw_parser.add_argument('-x', '--exclude', nargs="+",
                                 help='exclude these files from the report')
+    trackfw_parser.add_argument('-m', '--max', help='limit results', default=0, type=int)
     trackfw_parser.set_defaults(func=track_fws)
 
     rerun_fws_parser = subparsers.add_parser('rerun_fws', help='re-run Firework(s)')
@@ -648,6 +684,7 @@ def lpad():
     rerun_fws_parser.add_argument('-n', '--name', help='name')
     rerun_fws_parser.add_argument(*state_args, **state_kwargs)
     rerun_fws_parser.add_argument(*query_args, **query_kwargs)
+    rerun_fws_parser.add_argument(*launches_mode_args, **launches_mode_kwargs)
     rerun_fws_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(PW_CHECK_NUM))
     rerun_fws_parser.add_argument('--task-level', action='store_true', help='Enable task level recovery')
     rerun_fws_parser.add_argument('-lid', '--launch_id', nargs='+',
@@ -664,6 +701,7 @@ def lpad():
     defuse_fw_parser.add_argument('-n', '--name', help='name')
     defuse_fw_parser.add_argument(*state_args, **state_kwargs)
     defuse_fw_parser.add_argument(*query_args, **query_kwargs)
+    defuse_fw_parser.add_argument(*launches_mode_args, **launches_mode_kwargs)
     defuse_fw_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(PW_CHECK_NUM))
     defuse_fw_parser.set_defaults(func=defuse_fws)
 
@@ -672,6 +710,7 @@ def lpad():
     reignite_fw_parser.add_argument('-n', '--name', help='name')
     reignite_fw_parser.add_argument(*state_args, **state_kwargs)
     reignite_fw_parser.add_argument(*query_args, **query_kwargs)
+    reignite_fw_parser.add_argument(*launches_mode_args, **launches_mode_kwargs)
     reignite_fw_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(PW_CHECK_NUM))
     reignite_fw_parser.set_defaults(func=reignite_fws)
 
@@ -681,6 +720,7 @@ def lpad():
     update_fws_parser.add_argument('-n', '--name', help='get FWs with this name')
     update_fws_parser.add_argument(*state_args, **state_kwargs)
     update_fws_parser.add_argument(*query_args, **query_kwargs)
+    update_fws_parser.add_argument(*launches_mode_args, **launches_mode_kwargs)
     update_fws_parser.add_argument("-u", "--update", type=str,
                                    help='Doc update (enclose pymongo-style dict '
                                         'in single-quotes, e.g. \'{'
@@ -707,15 +747,8 @@ def lpad():
                                action="store_true")
     get_wf_parser.set_defaults(func=get_wfs)
 
-    display = subparsers.add_parser(
-            'display_wflows', help='Graphical display of Workflows')
-    display.add_argument(*fw_id_args, **fw_id_kwargs)
-    display.add_argument(
-        '-d', '--depth', help="Depth of links to search for.",
-        default=1, type=int)
-    display.set_defaults(func=display_wfs)
-
     defuse_wf_parser = subparsers.add_parser('defuse_wflows', help='cancel (de-fuse) an entire Workflow')
+    defuse_wf_parser.add_argument('--defuse_all_states', help='also defuse COMPLETED and FIZZLED workflows', action='store_true')
     defuse_wf_parser.add_argument(*fw_id_args, **fw_id_kwargs)
     defuse_wf_parser.add_argument('-n', '--name', help='name')
     defuse_wf_parser.add_argument(*state_args, **state_kwargs)
@@ -769,6 +802,7 @@ def lpad():
                                 type=int)
     fizzled_parser.add_argument('--fizzle', help='mark lost runs as fizzled', action='store_true')
     fizzled_parser.add_argument('--rerun', help='rerun lost runs', action='store_true')
+    fizzled_parser.add_argument('--refresh', help='refresh the detected inconsistent fireworks', action='store_true')
     fizzled_parser.add_argument('--max_runtime', help='max runtime, matching failures ran no longer than this (seconds)', type=int)
     fizzled_parser.add_argument('--min_runtime', help='min runtime, matching failures must have run at least this long (seconds)', type=int)
     fizzled_parser.set_defaults(func=detect_lostruns)
@@ -779,6 +813,7 @@ def lpad():
     priority_parser.add_argument('-n', '--name', help='name')
     priority_parser.add_argument(*state_args, **state_kwargs)
     priority_parser.add_argument(*query_args, **query_kwargs)
+    priority_parser.add_argument(*launches_mode_args, **launches_mode_kwargs)
     priority_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(PW_CHECK_NUM))
     priority_parser.set_defaults(func=set_priority)
 
@@ -792,15 +827,20 @@ def lpad():
     parser.add_argument('-s', '--silencer', help='shortcut to mute log messages', action='store_true')
 
     webgui_parser = subparsers.add_parser('webgui', help='launch the web GUI')
-    webgui_parser.add_argument("--port", dest="port", type=int, default=5000,
-                        help="Port to run the server on (default: 5000)")
-    webgui_parser.add_argument("--host", dest="host", type=str, default="127.0.0.1",
-                        help="Host to run the server on (default: 127.0.0.1)")
+    webgui_parser.add_argument("--port", dest="port", type=int, default=WEBSERVER_PORT,
+                        help="Port to run the web server on (default: 5000 or WEBSERVER_PORT arg in FW_config.yaml)")
+    webgui_parser.add_argument("--host", dest="host", type=str, default=WEBSERVER_HOST,
+                        help="Host to run the web server on (default: 127.0.0.1 or WEBSERVER_HOST arg in FW_config.yaml)")
+    webgui_parser.add_argument('--debug', help='print debug messages', action='store_true')
     webgui_parser.add_argument('-s', '--server_mode', help='run in server mode (skip opening the browser)', action='store_true')
+    webgui_parser.add_argument('--fwquery', help='additional query filter for FireWorks as JSON string')
+    webgui_parser.add_argument('--wflowquery', help='additional query filter for Workflows as JSON string')
     webgui_parser.set_defaults(func=webgui)
 
     recover_parser = subparsers.add_parser('recover_offline', help='recover offline workflows')
     recover_parser.add_argument('-i', '--ignore_errors', help='ignore errors', action='store_true')
+    recover_parser.add_argument('-w', '--fworker_file', help='path to fworker file. An empty string will match all the workers', default=FWORKER_LOC)
+    recover_parser.add_argument('-pe', '--print-errors', help='print errors', action='store_true')
     recover_parser.set_defaults(func=recover_offline)
 
     forget_parser = subparsers.add_parser('forget_offline', help='forget offline workflows')
@@ -833,41 +873,27 @@ def lpad():
     refresh_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(PW_CHECK_NUM))
     refresh_parser.set_defaults(func=refresh)
 
-    report_parser = subparsers.add_parser('report', help='Various statistics, type "lpad report -h" for more.',
-                    parents=[parent_parser])
-    report_parent_parser=ArgumentParser(add_help=False)
-    report_parent_parser.add_argument('-s', '--start', help="The start time (inclusive) to query in isoformat (YYYY-MM-DDTHH:MM:SS.mmmmmm). Default is 30 days from now.")
-    report_parent_parser.add_argument('-e', '--end', help="The end time (exclusive) to query in isoformat (YYYY-MM-DDTHH:MM:SS.mmmmmm). Default is now.")
-    report_parent_parser.add_argument('-w', '--weeks', help="Time difference in weeks to calculate start time from end time.", type=float, default=0)
-    report_parent_parser.add_argument('-d', '--days', help="Time difference in days to calculate start time from end time.", type=float, default=30)
-    report_parent_parser.add_argument('-o', '--hours', help="Time difference in hours to calculate start time from end time.", type=float, default=0)
-    report_parent_parser.add_argument('-m', '--minutes', help="Time difference in minutes to calculate start time from end time.", type=float, default=0)
-    report_parent_parser.add_argument('-q', '--query', help="Additional Pymongo queries to filter entries for process.")
-    report_subparser = report_parser.add_subparsers(title="action", dest="action_command")
+    unlock_parser = admin_subparser.add_parser('unlock', help='manually unlock a workflow that is locked (only if you know what you are doing!)')
+    unlock_parser.add_argument(*fw_id_args, **fw_id_kwargs)
+    unlock_parser.add_argument('-n', '--name', help='name')
+    unlock_parser.add_argument(*state_args, **state_kwargs)
+    unlock_parser.add_argument(*query_args, **query_kwargs)
+    unlock_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(PW_CHECK_NUM))
+    unlock_parser.set_defaults(func=unlock)
 
-    fw_report_parser = report_subparser.add_parser('fws', help='Get a report for fireworks', parents=[report_parent_parser])
-    fw_report_parser.add_argument('-f', '--time_field', default="time_end")
-    fw_report_parser.set_defaults(func=report)
-    launch_report_parser = report_subparser.add_parser('launches', help='Get a report for luanches', parents=[report_parent_parser])
-    launch_report_parser.add_argument('-f', '--time_field', default="time_end")
-    launch_report_parser.add_argument('-r', '--runtime_stats', action="store_true")
-    launch_report_parser.add_argument('-i', '--include_ids', action="store_true")
-    launch_report_parser.set_defaults(func=report)
-    wf_report_parser = report_subparser.add_parser('wfs', help='Get a report for workflows', parents=[report_parent_parser])
-    wf_report_parser.add_argument('-f', '--time_field', default="updated_on")
-    wf_report_parser.set_defaults(func=report)
-    daily_report_parser = report_subparser.add_parser('daily', help='Get a daily report for fireworks', parents=[report_parent_parser])
-    daily_report_parser.add_argument("-f", '--time_field', default="time_end")
-    daily_report_parser.set_defaults(func=report)
-    group_fizzled_fw_parser = report_subparser.add_parser('group_fizzled_fws', help='Group fizzled fireworks', parents=[report_parent_parser])
-    group_fizzled_fw_parser.add_argument("group_by", help="Database field used to group fireworks items. For example: 'spec.task_type'")
-    group_fizzled_fw_parser.add_argument("-i", '--include_ids', action="store_true")
-    group_fizzled_fw_parser.set_defaults(func=report)
-    identify_catastrophes_parser = report_subparser.add_parser('catastrophes', help='Get days with higher failure ratio', parents=[report_parent_parser])
-    identify_catastrophes_parser.add_argument('-t', '--error_ratio', type=float, default=0.01)
-    identify_catastrophes_parser.add_argument('-i', '--include_ids', action="store_true")
-    identify_catastrophes_parser.add_argument('-r', '--runtime_stats', action="store_true")
-    identify_catastrophes_parser.set_defaults(func=report)
+    report_parser = subparsers.add_parser('report', help='Compile a report of runtime stats, type "lpad report -h" for more options.')
+    report_parser.add_argument("-c", "--collection", help="The collection to report on; choose from 'fws' (default), 'wflows', or 'launches'.", default="fws")
+    report_parser.add_argument('-i', '--interval', help="Interval on which to split the report. Choose from 'minutes', 'hours', 'days' (default), 'months', or 'years'.", default="days")
+    report_parser.add_argument("-n", "--num_intervals", help="The number of intervals on which to report (default=5)", type=int, default=5)
+    report_parser.add_argument('-q', '--query', help="Additional Pymongo queries to filter entries before processing.")
+    report_parser.set_defaults(func=report)
+
+    introspect_parser = subparsers.add_parser('introspect', help='Introspect recent runs to pin down errors')
+    introspect_parser.add_argument('-m', '--max', help='examine past <max> results', default=100, type=int)
+    introspect_parser.add_argument('-t', '--threshold',
+                                   help='controls signal to noise ratio, e.g., 10 means difference of at least 10 runs between fizzled/completed count',
+                                   default=10, type=int)
+    introspect_parser.set_defaults(func=introspect)
 
     args = parser.parse_args()
 
