@@ -1,3 +1,7 @@
+# coding: utf-8
+
+from __future__ import unicode_literals
+
 """
 A runnable script for launching rockets (a command-line interface to queue_launcher.py)
 """
@@ -42,12 +46,13 @@ def do_launch(args):
     args.loglvl = 'CRITICAL' if args.silencer else args.loglvl
 
     if args.command == 'rapidfire':
-        rapidfire(launchpad, fworker, queueadapter, args.launch_dir,
-                  args.nlaunches, args.maxjobs_queue,
-                  args.maxjobs_block, args.sleep, args.reserve, args.loglvl)
+        rapidfire(launchpad, fworker=fworker, qadapter=queueadapter, launch_dir=args.launch_dir,
+                  nlaunches=args.nlaunches, njobs_queue=args.maxjobs_queue,
+                  njobs_block=args.maxjobs_block, sleep_time=args.sleep,
+                  reserve=args.reserve, strm_lvl=args.loglvl, timeout=args.timeout, fill_mode=args.fill_mode)
     else:
         launch_rocket_to_queue(launchpad, fworker, queueadapter,
-                               args.launch_dir, args.reserve, args.loglvl)
+                               args.launch_dir, args.reserve, args.loglvl, False, args.fill_mode)
 
 def qlaunch():
     m_description = 'This program is used to submit jobs to a queueing system. Details of the job and queue \
@@ -104,14 +109,16 @@ def qlaunch():
     parser.add_argument('-c', '--config_dir',
                         help='path to a directory containing the config file (used if -l, -w, -q unspecified)',
                         default=CONFIG_FILE_DIR)
+    parser.add_argument('-f', '--fill_mode', help='launch queue submissions even when there is nothing to run', action='store_true')
 
     rapid_parser.add_argument('-m', '--maxjobs_queue',
-                              help='maximum jobs to keep in queue for this user', default=10,
+                              help='maximum jobs to keep in queue for this user', default=0,
                               type=int)
     rapid_parser.add_argument('-b', '--maxjobs_block',
                               help='maximum jobs to put in a block',
                               default=500, type=int)
     rapid_parser.add_argument('--nlaunches', help='num_launches (int or "infinite"; default 0 is all jobs in DB)', default=0)
+    rapid_parser.add_argument('--timeout', help='timeout (secs) after which to quit (default None)', default=None, type=int)
     rapid_parser.add_argument('--sleep', help='sleep time between loops', default=None, type=int)
 
     args = parser.parse_args()
@@ -173,4 +180,4 @@ def qlaunch():
             break
 
 if __name__ == '__main__':
-    qlaunch()
\ No newline at end of file
+    qlaunch()
