@@ -1,3 +1,7 @@
+# coding: utf-8
+
+from __future__ import unicode_literals
+
 """
 A runnable script to launch a single Rocket (a command-line interface to rocket_launcher.py)
 """
@@ -40,6 +44,8 @@ def rlaunch():
     single_parser.add_argument('--offline', help='run in offline mode (FW.json required)', action='store_true')
 
     rapid_parser.add_argument('--nlaunches', help='num_launches (int or "infinite"; default 0 is all jobs in DB)', default=0)
+    rapid_parser.add_argument('--timeout', help='timeout (secs) after which to quit (default None)', default=None, type=int)
+    rapid_parser.add_argument('--max_loops', help='after this many sleep loops, quit even in infinite nlaunches mode (default -1 is infinite loops)', default=-1, type=int)
     rapid_parser.add_argument('--sleep', help='sleep time between loops (secs)', default=None, type=int)
 
     parser.add_argument('-l', '--launchpad_file', help='path to launchpad file', default=LAUNCHPAD_LOC)
@@ -52,7 +58,7 @@ def rlaunch():
 
     args = parser.parse_args()
 
-    signal.signal(signal.SIGINT, handle_interrupt)  # graceful exist on ^C
+    signal.signal(signal.SIGINT, handle_interrupt)  # graceful exit on ^C
 
     if not args.launchpad_file and os.path.exists(os.path.join(args.config_dir, 'my_launchpad.yaml')):
         args.launchpad_file = os.path.join(args.config_dir, 'my_launchpad.yaml')
@@ -79,10 +85,12 @@ def rlaunch():
     get_my_ip()
 
     if args.command == 'rapidfire':
-        rapidfire(launchpad, fworker, None, args.nlaunches, -1, args.sleep, args.loglvl)
+        rapidfire(launchpad, fworker=fworker, m_dir=None, nlaunches=args.nlaunches,
+                  max_loops=args.max_loops, sleep_time=args.sleep, strm_lvl=args.loglvl,
+                  timeout=args.timeout)
 
     else:
         launch_rocket(launchpad, fworker, args.fw_id, args.loglvl)
 
 if __name__ == '__main__':
-    rlaunch()
\ No newline at end of file
+    rlaunch()
