@@ -1,6 +1,7 @@
 # coding: utf-8
 
 from __future__ import unicode_literals
+import json
 
 from multiprocessing import Pool
 import os
@@ -16,13 +17,14 @@ from fireworks.core.launchpad import LaunchPad, WFLock
 from fireworks.core.rocket_launcher import launch_rocket, rapidfire
 from fireworks.features.background_task import BackgroundTask
 from fireworks.fw_config import WFLOCK_EXPIRATION_KILL
+from fireworks.queue.queue_launcher import setup_offline_job
 from fireworks.user_objects.dupefinders.dupefinder_exact import DupeFinderExact
 from fireworks.user_objects.firetasks.fileio_tasks import FileTransferTask, FileWriteTask
 from fireworks.user_objects.firetasks.script_task import ScriptTask, PyTask
 from fireworks.user_objects.firetasks.templatewriter_task import TemplateWriterTask
 from fw_tutorials.dynamic_wf.fibadd_task import FibonacciAdderTask
 from fw_tutorials.firetask.addition_task import AdditionTask
-from fireworks.tests.tasks import DummyTask
+from fireworks.tests.tasks import DummyFWEnvTask, DummyJobPassTask, DummyLPTask
 from fireworks.features.stats import FWStats
 import six
 
@@ -37,13 +39,6 @@ TESTDB_NAME = 'fireworks_unittest'
 MODULE_DIR = os.path.dirname(os.path.abspath(__file__))
 NCORES_PARALLEL_TEST = 4
 
-#TODO: make these tests much better. Right now they are just a crude first line of defense.
-
-# TODO: add dupefinder tests
-# TODO: test serialization of YAML on at least one WF
-
-# TODO: cleanup!!
-
 def random_launch(lp_creds):
         lp = LaunchPad.from_dict(lp_creds)
         while lp.run_exists(None):
@@ -100,12 +95,17 @@ class MongoTests(unittest.TestCase):
                 os.remove(f)
 
     def setUp(self):
+        self.lp.reset(password=None, require_password=False)
         self.old_wd = os.getcwd()
 
     def tearDown(self):
         self.lp.reset(password=None, require_password=False)
         if os.path.exists(os.path.join('FW.json')):
             os.remove('FW.json')
+        if os.path.exists(os.path.join('FW_offline.json')):
+            os.remove('FW_offline.json')
+        if os.path.exists(os.path.join('FW_ping.json')):
+            os.remove('FW_ping.json')
         os.chdir(self.old_wd)
         for i in glob.glob(os.path.join(MODULE_DIR, 'launcher*')):
             shutil.rmtree(i)
@@ -119,6 +119,78 @@ class MongoTests(unittest.TestCase):
         self.assertEqual(self.lp.get_launch_by_id(1).action.stored_data[
             'stdout'], 'test1\n')
 
+    def test_basic_fw_offline(self):
+        test1 = ScriptTask.from_str("python -c 'print(\"test1\")'",
+                                    {'store_stdout': True})
+        fw = Firework(test1)
+        self.lp.add_wf(fw)
+
+        fw, launch_id = self.lp.reserve_fw(self.fworker, os.getcwd())
+
+        setup_offline_job(self.lp, fw, launch_id)
+
+        launch_rocket(None, self.fworker)
+
+        with open(os.path.join(os.getcwd(), "FW_offline.json")) as f:
+            fwo = json.load(f)
+            self.assertEquals(fwo["state"], "COMPLETED")
+            self.assertEquals(fwo["launch_id"], 1)
+            self.assertEquals(fwo["fwaction"], {'update_spec': {}, 'mod_spec': [], 'stored_data': {'returncode': 0, 'stdout': u'test1\n', 'all_returncodes': [0]}, 'exit': False, 'detours': [], 'additions': [], 'defuse_children': False, 'defuse_workflow': False})
+
+        with open(os.path.join(os.getcwd(), "FW_ping.json")) as f:
+            fwp = json.load(f)
+            self.assertIsNotNone(fwp["ping_time"])
+
+        l = self.lp.offline_runs.find_one({"completed": False, "deprecated": False}, {"launch_id": 1})
+        self.lp.recover_offline(l['launch_id'])
+        self.assertEqual(self.lp.get_launch_by_id(1).action.stored_data['stdout'], 'test1\n')
+
+    def test_offline_fw_passinfo(self):
+        fw1 = Firework([AdditionTask()], {"input_array": [1,1]}, name="1")
+        fw2 = Firework([AdditionTask()], {"input_array": [2,2]}, name="2")
+        fw3 = Firework([AdditionTask()], {"input_array": [3]}, parents=[fw1, fw2], name="3")
+
+        wf = Workflow([fw1, fw2, fw3])
+        self.lp.add_wf(wf)
+
+        # make dirs for launching jobs
+        cur_dir = os.path.dirname(os.path.abspath(__file__))
+
+        os.mkdir(os.path.join(cur_dir, "launcher_1"))
+        os.mkdir(os.path.join(cur_dir, "launcher_2"))
+        os.mkdir(os.path.join(cur_dir, "launcher_3"))
+
+        # launch two parent jobs
+        os.chdir(os.path.join(cur_dir, "launcher_1"))
+        fw, launch_id = self.lp.reserve_fw(self.fworker, os.getcwd())
+        setup_offline_job(self.lp, fw, launch_id)
+        launch_rocket(None, self.fworker)
+
+        os.chdir(os.path.join(cur_dir, "launcher_2"))
+        fw, launch_id = self.lp.reserve_fw(self.fworker, os.getcwd())
+        setup_offline_job(self.lp, fw, launch_id)
+        launch_rocket(None, self.fworker)
+
+        # recover jobs
+        for l in self.lp.offline_runs.find({"completed": False, "deprecated": False}, {"launch_id": 1}):
+            fw = self.lp.recover_offline(l['launch_id'])
+
+        # launch child job
+        os.chdir(os.path.join(cur_dir, "launcher_3"))
+        fw, launch_id = self.lp.reserve_fw(self.fworker, os.getcwd())
+        last_fw_id = fw.fw_id
+        setup_offline_job(self.lp, fw, launch_id)
+        launch_rocket(None, self.fworker)
+
+        # recover jobs
+        for l in self.lp.offline_runs.find({"completed": False, "deprecated": False}, {"launch_id": 1}):
+            fw = self.lp.recover_offline(l['launch_id'])
+
+        # confirm the sum in the child job
+        child_fw = self.lp.get_fw_by_id(last_fw_id)
+        self.assertEqual(set(child_fw.spec['input_array']), set([2, 3, 4]))
+        self.assertEqual(child_fw.launches[0].action.stored_data["sum"], 9)
+
     def test_multi_fw(self):
         test1 = ScriptTask.from_str("python -c 'print(\"test1\")'",
                                     {'store_stdout': True})
@@ -144,7 +216,7 @@ class MongoTests(unittest.TestCase):
 
             # store workflow and launch it locally, single shot
             self.lp.add_wf(fw)
-            launch_rocket(self.lp, FWorker())
+            launch_rocket(self.lp, self.fworker)
 
             # read inputs.txt, words.txt, dest
             for d in [dest1, dest2]:
@@ -234,20 +306,85 @@ class MongoTests(unittest.TestCase):
         self.assertEqual(set(links[4]), set([2]))
         self.assertEqual(set(links[5]), set([2]))
 
-    def test_fworkerenv(self):
-        t = DummyTask()
+    def test_fw_env(self):
+        t = DummyFWEnvTask()
         fw = Firework(t)
         self.lp.add_wf(fw)
         launch_rocket(self.lp, self.fworker)
-        self.assertEqual(self.lp.get_launch_by_id(1).action.stored_data[
-                             'data'],
-                         "hello")
+        self.assertEqual(self.lp.get_launch_by_id(1).action.stored_data['data'], "hello")
         self.lp.add_wf(fw)
         launch_rocket(self.lp, FWorker(env={"hello": "world"}))
         self.assertEqual(self.lp.get_launch_by_id(2).action.stored_data[
                              'data'],
                          "world")
 
+    def test_job_info(self):
+        fw1 = Firework([ScriptTask.from_str('echo "Testing job info"')], spec={"_pass_job_info": True}, fw_id=1)
+        fw2 = Firework([DummyJobPassTask()], parents=[fw1], spec={"_pass_job_info": True, "target": 1}, fw_id=2)
+        fw3 = Firework([DummyJobPassTask()], parents=[fw2], spec={"target":2}, fw_id=3)
+        self.lp.add_wf(Workflow([fw1, fw2, fw3]))
+        launch_rocket(self.lp, self.fworker)
+
+        target_fw_id = self.lp.get_fw_ids({"spec.target": 1})[0]
+        modified_spec = self.lp.get_fw_by_id(target_fw_id).spec
+        """
+        cnt = 0
+        while '_job_info' not in modified_spec and cnt < 5:
+            print(modified_spec)
+            modified_spec = self.lp.get_fw_by_id(target_fw_id).spec
+            time.sleep(5)
+            cnt += 1
+        """
+
+        self.assertIsNotNone(modified_spec['_job_info'])
+        self.assertIsNotNone(modified_spec['_job_info'][0]["launch_dir"])
+        self.assertEqual(modified_spec['_job_info'][0]['name'], 'Unnamed FW')
+        self.assertEqual(modified_spec['_job_info'][0]['fw_id'], 1)
+
+        launch_rocket(self.lp, self.fworker)
+
+        target_fw_id = self.lp.get_fw_ids({"spec.target": 2})[0]
+        modified_spec = self.lp.get_fw_by_id(target_fw_id).spec
+
+        """
+        cnt = 0
+        while '_job_info' not in modified_spec and cnt < 5:
+            print(modified_spec)
+            modified_spec = self.lp.get_fw_by_id(target_fw_id).spec
+            time.sleep(5)
+            cnt += 1
+        """
+
+        self.assertEqual(len(modified_spec['_job_info']), 2)
+
+    def test_preserve_fworker(self):
+        fw1 = Firework([ScriptTask.from_str('echo "Testing preserve FWorker"')], spec={"_preserve_fworker": True}, fw_id=1)
+        fw2 = Firework([ScriptTask.from_str('echo "Testing preserve FWorker pt 2"')], spec={"target": 1}, parents=[fw1], fw_id=2)
+        self.lp.add_wf(Workflow([fw1, fw2]))
+        launch_rocket(self.lp, self.fworker)
+
+        target_fw_id = self.lp.get_fw_ids({"spec.target": 1})[0]
+
+        modified_spec = self.lp.get_fw_by_id(target_fw_id).spec
+
+        """
+        cnt = 0
+        while '_fworker' not in modified_spec and cnt < 5:
+            modified_spec = self.lp.get_fw_by_id(target_fw_id).spec
+            print(modified_spec)
+            time.sleep(5)
+            cnt += 1
+        """
+
+        self.assertIsNotNone(modified_spec['_fworker'])
+
+    def test_add_lp_and_fw_id(self):
+        fw1 = Firework([DummyLPTask()], spec={"_add_launchpad_and_fw_id": True})
+        self.lp.add_wf(fw1)
+        launch_rocket(self.lp, self.fworker)
+        self.assertEqual(self.lp.get_launch_by_id(1).action.stored_data['fw_id'], 1)
+        self.assertIsNotNone(self.lp.get_launch_by_id(1).action.stored_data['host'])
+
     def test_spec_copy(self):
         task1 = ScriptTask.from_str('echo "Task 1"')
         task2 = ScriptTask.from_str('echo "Task 2"')
@@ -297,7 +434,7 @@ class MongoTests(unittest.TestCase):
 
         self.assertEqual(self.lp.launches.count(), 1)
 
-    def test_add_wf_to_fws(self):
+    def test_append_wf(self):
         fw1 = Firework([UpdateSpecTask()])
         fw2 = Firework([ModSpecTask()])
         self.lp.add_wf(Workflow([fw1, fw2]))
@@ -308,7 +445,7 @@ class MongoTests(unittest.TestCase):
         self.assertEqual(self.lp.fireworks.count(), 3)  # due to detour
 
         new_wf = Workflow([Firework([ModSpecTask()])])
-        self.lp.add_wf_to_fws(new_wf, [1, 2])
+        self.lp.append_wf(new_wf, [1, 2])
         launch_rocket(self.lp, self.fworker)  # launch detour
         launch_rocket(self.lp, self.fworker)  # launch new FW
         launch_rocket(self.lp, self.fworker)  # dummy launch
@@ -320,21 +457,21 @@ class MongoTests(unittest.TestCase):
         self.assertEqual(self.lp.fireworks.count(), 4)
 
         new_wf = Workflow([Firework([ModSpecTask()])])
-        self.lp.add_wf_to_fws(new_wf, [4])
+        self.lp.append_wf(new_wf, [4])
         launch_rocket(self.lp, self.fworker)  # launch new FW
         new_fw = self.lp.get_fw_by_id(5)
         self.assertEqual(new_fw.spec['dummy2'], [True])
 
         new_wf = Workflow([Firework([ModSpecTask()])])
-        self.assertRaises(ValueError, self.lp.add_wf_to_fws, new_wf, [4], detour=True)
+        self.assertRaises(ValueError, self.lp.append_wf, new_wf, [4], detour=True)
 
-    def test_add_wf_to_fws_detour(self):
+    def test_append_wf_detour(self):
         fw1 = Firework([ModSpecTask()], fw_id=1)
         fw2 = Firework([ModSpecTask()], fw_id=2, parents=[fw1])
         self.lp.add_wf(Workflow([fw1, fw2]))
 
         new_wf = Workflow([Firework([ModSpecTask()])])
-        self.lp.add_wf_to_fws(new_wf, [1], detour=True)
+        self.lp.append_wf(new_wf, [1], detour=True)
 
         launch_rocket(self.lp, self.fworker)
         launch_rocket(self.lp, self.fworker)
@@ -346,10 +483,9 @@ class MongoTests(unittest.TestCase):
         test1 = ScriptTask.from_str("python -c 'print(\"test1\")'", {'store_stdout': True})
         fw = Firework(test1, {"_dupefinder": DupeFinderExact()}, fw_id=1)
         self.lp.add_wf(fw)
-        self.assertEqual(WFLOCK_EXPIRATION_KILL, True)
         # add a manual lock
         with WFLock(self.lp, 1):
-            with WFLock(self.lp, 1, expire_secs=1):
+            with WFLock(self.lp, 1, kill=True, expire_secs=1):
                 self.assertTrue(True)  # dummy to make sure we got here
 
     def test_fizzle(self):
