@@ -20,8 +20,21 @@ from fireworks.utilities.fw_utilities import explicit_serialize
 
 
 @explicit_serialize
-class DummyTask(FireTaskBase):
+class DummyFWEnvTask(FireTaskBase):
 
     def run_task(self, fw_spec):
         data = fw_spec["_fw_env"].get("hello", "hello")
         return FWAction(stored_data={"data": data})
+
+
+@explicit_serialize
+class DummyJobPassTask(FireTaskBase):
+
+    def run_task(self, fw_spec):
+        return FWAction(stored_data={"data": fw_spec['_job_info']})
+
+@explicit_serialize
+class DummyLPTask(FireTaskBase):
+
+    def run_task(self, fw_spec):
+        return FWAction(stored_data={"fw_id": self.fw_id, "host": self.launchpad.host})
\ No newline at end of file
