@@ -14,13 +14,12 @@ __date__ = 'Mar 01, 2013'
 
 class DupeFinderExact(DupeFinderBase):
     """
-    TODO: add docs
+    This DupeFinder requires an exact spec match between FireWorks.
     """
 
     _fw_name = 'DupeFinderExact'
 
     # TODO: move the logic into query() instead of verify() for better performance
-    # TODO: just make the 'prematch' thing be always a list of keys...no custom queries allowed (in theory)
 
     def verify(self, spec1, spec2):
         return spec1 == spec2
