@@ -7,6 +7,9 @@ import shutil
 import traceback
 
 from os.path import expandvars, expanduser, abspath
+
+import time
+
 from fireworks.core.firework import FireTaskBase
 from monty.shutil import compress_dir, decompress_dir
 
@@ -69,7 +72,10 @@ class FileTransferTask(FireTaskBase):
         - dest: (str) destination directory, if not specified within files parameter
     Optional params:
         - server: (str) server host for remote transfer
+        - user: (str) user to authenticate with on remote server
         - key_filename: (str) optional SSH key location for remote transfer
+        - max_retry: (int) number of times to retry failed transfers; defaults to `0` (no retries)
+        - retry_delay: (int) number of seconds to wait between retries; defaults to `10`
     """
     _fw_name = 'FileTransferTask'
     required_params = ["mode", "files"]
@@ -87,6 +93,8 @@ class FileTransferTask(FireTaskBase):
     def run_task(self, fw_spec):
         shell_interpret = self.get('shell_interpret', True)
         ignore_errors = self.get('ignore_errors')
+        max_retry = self.get('max_retry', 0)
+        retry_delay = self.get('retry_delay', 10)
         mode = self.get('mode', 'move')
 
         if mode == 'rtransfer':
@@ -95,7 +103,7 @@ class FileTransferTask(FireTaskBase):
             import paramiko
             ssh = paramiko.SSHClient()
             ssh.load_host_keys(expanduser(os.path.join("~", ".ssh", "known_hosts")))
-            ssh.connect(self['server'], key_filename=self.get['key_filename'])
+            ssh.connect(self['server'], username=self.get('user'), key_filename=self.get('key_filename'))
             sftp = ssh.open_sftp()
 
         for f in self["files"]:
@@ -116,7 +124,10 @@ class FileTransferTask(FireTaskBase):
                             if os.path.isfile(os.path.join(src,f)):
                                 sftp.put(os.path.join(src, f), os.path.join(dest, f))
                     else:
-                        sftp.put(src, dest)
+                        if not self._rexists(sftp, dest):
+                            sftp.mkdir(dest)
+
+                        sftp.put(src, os.path.join(dest, os.path.basename(src)))
 
                 else:
                     if 'dest' in f:
@@ -129,10 +140,18 @@ class FileTransferTask(FireTaskBase):
 
             except:
                 traceback.print_exc()
-                if not ignore_errors:
+                if max_retry:
+
+                    # we want to avoid hammering either the local or remote machine
+                    time.sleep(retry_delay)
+                    self['max_retry'] -= 1
+                    self.run_task(fw_spec)
+
+                elif not ignore_errors:
                     raise ValueError(
                         "There was an error performing operation {} from {} "
                         "to {}".format(mode, self["files"], self["dest"]))
+
         if mode == 'rtransfer':
             sftp.close()
             ssh.close()
