@@ -16,7 +16,6 @@ __maintainer__ = 'Anubhav Jain'
 __email__ = 'ajain@lbl.gov'
 __date__ = 'Feb 18, 2013'
 
-# TODO: document!
 # TODO: add maximum length of 10,000 chars for stored fields
 
 
@@ -39,8 +38,8 @@ class ScriptTask(FireTaskBase):
 
     def _run_task_internal(self, fw_spec, stdin):
         # run the program
-        stdout = subprocess.PIPE if self.store_stdout or self.stdout_file else sys.stdout
-        stderr = subprocess.PIPE if self.store_stderr or self.stderr_file else sys.stderr
+        stdout = subprocess.PIPE if self.store_stdout or self.stdout_file else None
+        stderr = subprocess.PIPE if self.store_stderr or self.stderr_file else None
         returncodes = []
         for s in self.script:
             p = subprocess.Popen(
@@ -132,7 +131,7 @@ class ScriptTask(FireTaskBase):
 class PyTask(FireTaskBase):
     _fw_name = 'PyTask'
 
-    """
+    __doc__ = """
     Runs any python function! Extremely powerful, which allows you to
     essentially run any accessible method on the system.
 
@@ -141,19 +140,18 @@ class PyTask(FireTaskBase):
             .copy, or some other function that is not part of the standard
             library!
         args (list): List of args. Default is empty.
+        kwargs (dict): Dictionary of keyword args. Default is empty.
+        auto_kwargs (bool): If True, all other params not starting with "_" are supplied as keyword args
         stored_data_varname (str): Whether to store the output in
             FWAction. If
             this is a string that does not evaluate to False, the output of
             the function will be stored as
             FWAction(stored_data={stored_data_varname: output}). The name is
             deliberately long to avoid potential name conflicts.
-
-        All other params not starting with "_" are supplied as keyword args
-        to the Python method.
     """
 
     required_params = ["func"]
-    optional_params = ["args", "stored_data_varname"]
+    optional_params = ["args", "kwargs",  "auto_kwargs", "stored_data_varname"]
 
     def run_task(self, fw_spec):
         toks = self["func"].rsplit(".", 1)
@@ -164,11 +162,16 @@ class PyTask(FireTaskBase):
         else:
             #Handle built in functions.
             func = getattr(builtins, toks[0])
+
         args = self.get("args", [])
-        kwargs = {k: v for k, v in self.items()
-                  if not (k.startswith("_") or k in ["func",
-                                                     "args",
-                                                     "stored_data_varname"])}
+        if self.get("auto_kwargs"):
+            kwargs = {k: v for k, v in self.items()
+                  if not (k.startswith("_") or k in self.required_params or k in self.optional_params)}
+        else:
+            kwargs = self.get("kwargs", {})
+
         output = func(*args, **kwargs)
-        if self.get("stored_data_varname"):
-            return FWAction(stored_data={self["stored_data_varname"]: output})
+        if isinstance(output,FWAction):
+            return output
+        elif self.get("stored_data_varname"):
+            return FWAction(stored_data={self["stored_data_varname"]: output})
\ No newline at end of file
