@@ -1,6 +1,7 @@
 # coding: utf-8
 
 from __future__ import unicode_literals
+import copy
 
 """
 This module implements a CommonAdaptor that supports standard PBS and SGE
@@ -8,6 +9,7 @@ queues.
 """
 import getpass
 import os
+import stat
 import re
 import subprocess
 from fireworks.queue.queue_adapter import QueueAdapterBase, Command
@@ -28,17 +30,20 @@ class CommonAdapter(QueueAdapterBase):
     TORQUE), SGE, and SLURM queues.
     """
     _fw_name = 'CommonAdapter'
-    supported_q_types = {
-        "PBS": "qsub",
-        "SGE": "qsub",
-        "SLURM": "sbatch",
-        "LoadLeveler": "llsubmit"
+
+    default_q_commands = {
+        "PBS": {"submit_cmd": "qsub", "status_cmd": "qstat"},
+        "SGE": {"submit_cmd": "qsub", "status_cmd": "qstat"},
+        "Cobalt": {"submit_cmd": "qsub", "status_cmd": "qstat"},
+        "SLURM": {"submit_cmd": "sbatch", "status_cmd": "squeue"},
+        "LoadLeveler": {"submit_cmd": "llsubmit", "status_cmd": "llq"},
+        "LoadSharingFacility": {"submit_cmd": "bsub", "status_cmd": "bjobs"}
     }
 
     def __init__(self, q_type, q_name=None, template_file=None, **kwargs):
         """
         :param q_type: The type of queue. Right now it should be either PBS,
-                       SGE, SLURM or LoadLeveler.
+                       SGE, SLURM, Cobalt or LoadLeveler.
         :param q_name: A name for the queue. Can be any string.
         :param template_file: The path to the template file. Leave it as
                               None (the default) to use Fireworks' built-in
@@ -46,17 +51,21 @@ class CommonAdapter(QueueAdapterBase):
                               on most queues.
         :param **kwargs: Series of keyword args for queue parameters.
         """
-        if q_type not in CommonAdapter.supported_q_types:
+        if q_type not in CommonAdapter.default_q_commands:
             raise ValueError(
                 "{} is not a supported queue type. "
                 "CommonAdaptor supports {}".format(
-                    q_type, list(CommonAdapter.supported_q_types.keys())))
+                    q_type, list(self.default_q_commands.keys())))
         self.q_type = q_type
         self.template_file = os.path.abspath(template_file) if template_file is not None else \
             CommonAdapter._get_default_template_file(q_type)
         self.q_name = q_name or q_type
         self.update(dict(kwargs))
 
+        self.q_commands = copy.deepcopy(CommonAdapter.default_q_commands)
+        if '_q_commands_override' in self:
+            self.q_commands[self.q_type].update(self["_q_commands_override"])
+
     def _parse_jobid(self, output_str):
         if self.q_type == "SLURM":
             for l in output_str.split("\n"):
@@ -65,9 +74,17 @@ class CommonAdapter(QueueAdapterBase):
         if self.q_type == "LoadLeveler":
             # Load Leveler: "llsubmit: The job "abc.123" has been submitted"
             re_string = r"The job \"(.*?)\" has been submitted"
+        elif self.q_type == "Cobalt":
+            # 99% of the time you just get:
+            # Cobalt: "199768"
+            # but there's a version that also includes project and queue
+            # information on preceeding lines and both of those  might 
+            # contain a number in any position.
+            re_string = r"(\b\d+\b)"
         else:
             # PBS: "1234.whatever",
             # SGE: "Your job 44275 ("jobname") has been submitted"
+            # Cobalt: "199768"
             re_string = r"(\d+)"
         m = re.search(re_string, output_str)
         if m:
@@ -75,20 +92,39 @@ class CommonAdapter(QueueAdapterBase):
         raise RuntimeError("Unable to parse jobid")
 
     def _get_status_cmd(self, username):
+        status_cmd = [self.q_commands[self.q_type]["status_cmd"]]
+
         if self.q_type == 'SLURM':
-            return ['squeue', '-o "%u"', '-u', username]
-        elif self.q_type == "LoadLeveler":
-            return ['llq', '-u', username]
+            # by default, squeue lists pending and running jobs
+            # -p: filter queue (partition)
+            # -h: no header line
+            # -o: reduce output to user only (shorter string to parse)
+            status_cmd.extend(['-o "%u"', '-u', username, '-h'])
+            if 'queue' in self and self['queue']:
+                status_cmd.extend(['-p', self['queue']])
+        elif self.q_type == "LoadSharingFacility":
+            #use no header and the wide format so that there is one line per job, and display only running and pending jobs
+            status_cmd.extend(['-p','-r','-o', 'jobID user queue', '-noheader', '-u', username])
+        elif self.q_type == "Cobalt":
+            header="JobId:User:Queue:Jobname:Nodes:Procs:Mode:WallTime:State:RunTime:Project:Location"
+            status_cmd.extend(['--header', header, '-u', username])
+        elif self.q_type == 'SGE':
+            status_cmd.extend(['-q', self['queue'], '-u', username])
         else:
-            return ['qstat', '-u', username]
+            status_cmd.extend(['-u', username])
+
+        return status_cmd
 
     def _parse_njobs(self, output_str, username):
         # TODO: what if username is too long for the output and is cut off?
 
-        if self.q_type == 'SLURM': # this special case might not be needed
-            # TODO: currently does not filter on queue name or job state
-            outs = output_str.split('\n')
-            return len([line.split() for line in outs if username in line])
+        # WRS: I may come back to this after confirming that Cobalt
+        #      strictly follows the PBS standard and replace the spliting
+        #      with a regex that would solve length issues
+
+        if self.q_type == 'SLURM':
+            # subtract one due to trailing '\n' and split behavior
+            return len(output_str.split('\n'))-1
 
         if self.q_type == "LoadLeveler":
             if 'There is currently no job status to report' in output_str:
@@ -96,10 +132,20 @@ class CommonAdapter(QueueAdapterBase):
             else:
                 # last line is: "1 job step(s) in query, 0 waiting, ..."
                 return int(output_str.split('\n')[-2].split()[0])
+        if self.q_type == "LoadSharingFacility":
+            #Just count the number of lines
+            return len(output_str.split('\n'))
+        if self.q_type == "SGE":
+            # want only lines that include username;
+            # this will exclude e.g. header lines
+            return len([l for l in output_str.split('\n') if username in l])
 
         count = 0
         for l in output_str.split('\n'):
             if l.lower().startswith("job"):
+                if self.q_type == "Cobalt":
+                    # Cobalt capitalzes headers
+                    l=l.lower()
                 header = l.split()
                 if self.q_type == "PBS":
                     #PBS has a ridiculous two word "Job ID" in header
@@ -131,12 +177,21 @@ class CommonAdapter(QueueAdapterBase):
                     script_file))
 
         queue_logger = self.get_qlogger('qadapter.{}'.format(self.q_name))
-        submit_cmd = CommonAdapter.supported_q_types[self.q_type]
+        submit_cmd = self.q_commands[self.q_type]["submit_cmd"]
         # submit the job
         try:
+            if self.q_type == "Cobalt":
+                # Cobalt requires scripts to be executable
+                os.chmod(script_file,stat.S_IRWXU|stat.S_IRGRP|stat.S_IXGRP)
             cmd = [submit_cmd, script_file]
-            p = subprocess.Popen(cmd, stdout=subprocess.PIPE,
-                                 stderr=subprocess.PIPE)
+            #For most of the queues handled by common_adapter, it's best to simply submit the file name
+            #as an argument.  LoadSharingFacility doesn't handle the header section (queue name, nodes, etc)
+            #when taking file arguments, so the file needs to be passed as stdin to make it work correctly.
+            if self.q_type == 'LoadSharingFacility':
+                with open(script_file, 'r') as inputFile:
+                    p = subprocess.Popen([submit_cmd],stdin=inputFile,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
+            else:
+                p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
             p.wait()
 
             # grab the returncode. PBS returns 0 if the job was successful
