@@ -1,7 +1,3 @@
-"""
-TODO: Modify module doc.
-"""
-
 from __future__ import division
 
 __author__ = "Shyue Ping Ong"
