@@ -1,8 +1,9 @@
 # coding: utf-8
 
 from __future__ import unicode_literals
+from monty.json import MontyDecoder, MSONable
 
-"""
+__doc__ = """
 This module aids in serializing and deserializing objects.
 
 To serialize a FW object, refer to the documentation for the FWSerializable class. To de-serialize an object,
@@ -25,9 +26,8 @@ Some advantages:
     - In some cases, objects can be serialized/deserialized extremely concisely, by use of only their fw_name (if no
     parameters are needed to describe the object)
 
-A dict created using FWSerializer's to_dict() method should be readable by Pymatgen's PMGDecoder,
-when the serialize_fw() decorator is used.
 """
+
 import traceback
 import pkgutil
 import inspect
@@ -45,7 +45,7 @@ except ImportError:
     from yaml import Loader as Loader, SafeDumper as Dumper
 import six
 
-from fireworks.fw_config import FW_NAME_UPDATES, YAML_STYLE, USER_PACKAGES
+from fireworks.fw_config import FW_NAME_UPDATES, YAML_STYLE, USER_PACKAGES, DECODE_MONTY, ENCODE_MONTY
 
 __author__ = 'Anubhav Jain'
 __copyright__ = 'Copyright 2012, The Materials Project'
@@ -66,7 +66,7 @@ def recursive_dict(obj, preserve_unicode=True):
     if obj is None:
         return None
 
-    if hasattr(obj, 'as_dict'):  # strictly for pymatgen compatibility
+    if ENCODE_MONTY and hasattr(obj, 'as_dict'):  # compatible with new monty JSONEncoder (MontyEncoder)
         return recursive_dict(obj.as_dict(), preserve_unicode)
 
     if hasattr(obj, 'to_dict'):
@@ -95,12 +95,16 @@ def _recursive_load(obj):
     if obj is None:
         return None
 
-    if hasattr(obj, '_fw_name'):
+    if hasattr(obj, '_fw_name') or isinstance(obj, MSONable):
         return obj
 
     if isinstance(obj, dict):
         if '_fw_name' in obj:
             return load_object(obj)
+
+        if DECODE_MONTY and '@module' in obj and '@class' in obj:  # MontyDecoder compatibility
+            return json.loads(json.dumps(obj), cls=MontyDecoder)
+
         return {k: _recursive_load(v) for k, v in obj.items()}
 
     if isinstance(obj, (list, tuple)):
@@ -194,11 +198,19 @@ class FWSerializable(object):
     def to_db_dict(self):
         return self.to_dict()
 
+    def as_dict(self):
+        # strictly for pseudo-compatibility with MSONable
+        # Note that FWSerializable is not MSONable, it uses _fw_name instead of __class__ and __module__
+        return self.to_dict()
+
     @classmethod
     @abc.abstractmethod
     def from_dict(cls, m_dict):
         raise NotImplementedError('FWSerializable object did not implement from_dict()!')
 
+    def __repr__(self):
+        return json.dumps(self.to_dict(), default=DATETIME_HANDLER)
+
     def to_format(self, f_format='json', **kwargs):
         """
         returns a String representation in the given format
